@@ -2,7 +2,7 @@
  *  Hamlib Alinco backend - DX77 description
  *  Copyright (c) 2001-2003 by Stephane Fillod
  *
- *	$Id: dx77.c,v 1.6 2003/04/06 18:40:35 fillods Exp $
+ *	$Id: dx77.c,v 1.10 2004/02/15 00:30:29 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -24,7 +24,10 @@
 #include "config.h"
 #endif
 
+#include <stdlib.h>
+
 #include <hamlib/rig.h>
+#include "idx_builtin.h"
 #include "alinco.h"
 
 
@@ -34,7 +37,7 @@
 
 #define DX77_FUNC (RIG_FUNC_FAGC|RIG_FUNC_NB|RIG_FUNC_TONE|RIG_FUNC_COMP)
 
-#define DX77_LEVEL_ALL (RIG_LEVEL_SQLSTAT|RIG_LEVEL_STRENGTH|RIG_LEVEL_RFPOWER|RIG_LEVEL_KEYSPD|RIG_LEVEL_BKINDL|RIG_LEVEL_CWPITCH)
+#define DX77_LEVEL_ALL (RIG_LEVEL_SQLSTAT|RIG_LEVEL_RAWSTR|RIG_LEVEL_RFPOWER|RIG_LEVEL_KEYSPD|RIG_LEVEL_BKINDL|RIG_LEVEL_CWPITCH)
 
 #define DX77_PARM_ALL (RIG_PARM_BEEP|RIG_PARM_BACKLIGHT)
 
@@ -57,10 +60,6 @@
 		{ 216, 60 }, \
 	} }
 
-static const struct alinco_priv_caps dx77_priv_caps = {
-		DX77_STR_CAL
-};
-
 /*
  * dx77 rig capabilities.
  *
@@ -80,9 +79,9 @@ const struct rig_caps dx77_caps = {
 .rig_model =  RIG_MODEL_DX77,
 .model_name = "DX-77",
 .mfg_name =  "Alinco",
-.version =  "0.1",
+.version =  "0.3",
 .copyright =  "LGPL",
-.status =  RIG_STATUS_UNTESTED,
+.status =  RIG_STATUS_BETA,
 .rig_type =  RIG_TYPE_TRANSCEIVER,
 .ptt_type =  RIG_PTT_NONE,
 .dcd_type =  RIG_DCD_NONE,
@@ -104,7 +103,9 @@ const struct rig_caps dx77_caps = {
 .has_set_level =  RIG_LEVEL_SET(DX77_LEVEL_ALL),
 .has_get_parm =  DX77_PARM_ALL,
 .has_set_parm =  RIG_PARM_SET(DX77_PARM_ALL),
-.level_gran =  {},                 /* FIXME: granularity */
+.level_gran = {
+	[LVL_RAWSTR] = { .min = { .i = 0 }, .max = { .i = 255 } },
+},
 .parm_gran =  {},
 .ctcss_list =  common_ctcss_list,
 .dcs_list =  NULL,
@@ -157,11 +158,12 @@ const struct rig_caps dx77_caps = {
         /* mode/filter list, remember: order matters! */
 .filters =  {
 		{RIG_MODE_SSB|RIG_MODE_CW, kHz(2.7)},
+		{RIG_MODE_CW, kHz(0.5)},
 		{RIG_MODE_AM|RIG_MODE_FM, kHz(8)},
 		{RIG_MODE_AM, kHz(2.7)},
 		RIG_FLT_END,
 	},
-.priv =  (void*)&dx77_priv_caps,
+.str_cal = DX77_STR_CAL,
 
 .set_freq =  alinco_set_freq,
 .get_freq =  alinco_get_freq,
