@@ -2,7 +2,7 @@
  *  Hamlib Dummy backend - main file
  *  Copyright (c) 2001-2003 by Stephane Fillod
  *
- *	$Id: dummy.c,v 1.34 2003/04/16 22:30:39 fillods Exp $
+ *	$Id: dummy.c,v 1.38 2004/01/15 22:43:59 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
@@ -983,9 +984,16 @@ static int dummy_send_morse(RIG *rig, vfo_t vfo, const char *msg)
  * Dummy rig capabilities.
  */
 
+#if 0
 #define DUMMY_FUNC  0x7fffffffffffffLL	/* has it all */
 #define DUMMY_LEVEL 0x7ffffffffffffffLL
 #define DUMMY_PARM  0x7ffffffffffffffLL
+#else
+#define DUMMY_FUNC  ((setting_t)-1)	/* has it all */
+#define DUMMY_LEVEL ((setting_t)-1)
+#define DUMMY_PARM  ((setting_t)-1)
+#endif
+
 #define DUMMY_VFO_OP  0x7ffffffL
 #define DUMMY_SCAN	0x7ffffffL
 
@@ -1038,7 +1046,7 @@ const struct rig_caps dummy_caps = {
   .has_set_level =  RIG_LEVEL_SET(DUMMY_LEVEL),
   .has_get_parm = 	 DUMMY_PARM,
   .has_set_parm = 	 RIG_PARM_SET(DUMMY_PARM),
-  .level_gran =		{ [LVL_CWPITCH] = 10 },
+  .level_gran =		 { [LVL_CWPITCH] = { .step = { .i = 10 } } },
   .ctcss_list = 	 common_ctcss_list,
   .dcs_list =   	 full_dcs_list,
   .chan_list = 	 {
