@@ -3,7 +3,7 @@
  *  Hamlib GNUradio backend - main file
  *  Copyright (c) 2001-2003 by Stephane Fillod
  *
- *	$Id: gnuradio.cc,v 1.4 2003/04/06 18:50:21 fillods Exp $
+ *	$Id: gnuradio.cc,v 1.8 2004/02/08 20:27:58 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -30,35 +30,35 @@
  */
 
 #include <VrSigSource.h>
-#include <VrAudioSource.h>
+#include <GrAudioSource.h>
 #include <VrFileSource.h>
 #include <VrNullSink.h>
-#include <VrAudioSink.h>
+#include <GrAudioSink.h>
 #include <VrFileSink.h>
 
+#include <make_GrMC4020Source.h>
 #include <VrFixOffset.h>
-#include <GrMC4020Source.h>
+#include <GrConvertSF.h>
 
 #include <VrConnect.h>
 #include <VrMultiTask.h>
-#include <GrFreqXlatingFIRfilterSCF.h>
-#include <GrFreqXlatingFIRfilterCCF.h>
-#include <GrFIRfilterFSF.h>
-#include <GrFIRfilterFFF.h>
-#include <VrQuadratureDemod.h>	/* FM */
-//#include <VrAmplitudeDemod.h>	/* AM */
-/* SSB mod */
-//#include <GrSSBMod.h>
-//#include <GrHilbert.h>
 
-#include <gr_firdes.h>
-#include <gr_fir_builderF.h>
+
+/* Demodulator chains */
+#include <nfm.h>
+#include <am.h>
+#include <ssb.h>
+#include <wfm.h>
+
 
 
 /* Chirp param's */
 #define CARRIER_FREQ	          	        1.070e6	// AM 1070
 #define AMPLITUDE				3000
 
+#define AUDIO_SINK	"/dev/dsp"
+#define AUDIO_SRC	"/dev/dsp1"
+
 #include <stdlib.h>
 #include <stdio.h>   /* Standard input/output definitions */
 #include <string.h>  /* String function definitions */
@@ -72,6 +72,7 @@
 #include <hamlib/rig.h>
 #include <misc.h>
 #include <token.h>
+#include <register.h>
 
 #include "gnuradio.h"
 #include "gr_priv.h"	// struct gnuradio_priv_data
@@ -141,7 +142,6 @@ int gr_init(RIG *rig)
   priv->m = NULL;
   priv->sink = NULL;
   priv->source = NULL;
-  priv->need_fixer = 0;
 
   for (i=0; i<NUM_CHAN; i++) {
   	init_chan(rig, RIG_VFO_N(i), &priv->chans[i]);
@@ -264,37 +264,41 @@ int gr_open(RIG *rig)
    * 	and override available modes with gnuradio's
    */
 
+  rig->state.has_set_func  |= priv->tuner->state.has_set_func;
+  rig->state.has_get_func  |= priv->tuner->state.has_get_func;
+  rig->state.has_set_level |= priv->tuner->state.has_set_level;
+  rig->state.has_get_level |= priv->tuner->state.has_get_level;
+  rig->state.has_set_parm  |= priv->tuner->state.has_set_parm;
+  rig->state.has_get_parm  |= priv->tuner->state.has_get_parm;
+
 
   /* ** Source ** */
   
   // --> short
 
-  priv->source = new VrFileSource<short>(priv->input_rate, "/tmp/fm95_5_half.dat", true);
+#if 0
+  if (!priv->source) {
+  	priv->source = new VrFileSource<short>(priv->input_rate, "/tmp/fm95_5_half.dat", true);
+  }
   // Chirp
   if (!priv->source)
   	priv->source = new VrSigSource<IOTYPE>(priv->input_rate, VR_SIN_WAVE, CARRIER_FREQ, AMPLITUDE);
+#endif
   //new VrChirpSource<IOTYPE>(priv->input_rate, AMPLITUDE, 4);
   /* VrFileSource (double sampling_freq, const char *file, bool repeat = false) */
   //priv->source = new VrFileSource<short>(priv->input_rate, "microtune_source.sw", true);
 
-  // short --> short
-  priv->need_fixer = 1;
-  priv->offset_fixer = new VrFixOffset<short,short>();
-  NWO_CONNECT (priv->source, priv->offset_fixer);
-
-  priv->sink = new VrAudioSink<short>();
+  priv->sink = new GrAudioSink<float>(1,AUDIO_SINK);
 
   /* ** Sink ** */
   if (!priv->sink)
-  	priv->sink = new VrNullSink<short>();
+  	priv->sink = new VrNullSink<float>();
   //priv->sink = new VrFileSink<short>("microtune_audio.sw");
 
   priv->m = new VrMultiTask ();
 
   priv->m->start();
 
-  /* or set it to MODE_NONE? */
-  //gr_set_mode(rig, RIG_VFO_CURR, RIG_MODE_WFM, RIG_PASSBAND_NORMAL);
   if (priv->tuner_model == RIG_MODEL_DUMMY) {
 	  gr_set_freq(rig, RIG_VFO_CURR, priv->IF_center_freq);
   }
@@ -323,13 +327,25 @@ int mc4020_open(RIG *rig)
 	struct gnuradio_priv_data *priv = (struct gnuradio_priv_data*)rig->state.priv;
 
 	/* input sample rate from PCI-DAS4020/12: 20000000  */
-	priv->source = new GrMC4020Source<short>(priv->input_rate, MCC_CH3_EN | MCC_ALL_1V);
+
+	priv->mc4020_source = make_GrMC4020SourceS(priv->input_rate, MCC_CH3_EN | MCC_ALL_1V);
+
+  	// short --> short
+  	priv->offset_fixer = new VrFixOffset<short,short>();
+
+  	// short --> float
+	priv->convert_SF = new GrConvertSF();
+
+	NWO_CONNECT (priv->mc4020_source, priv->offset_fixer);
+	NWO_CONNECT (priv->offset_fixer, priv->convert_SF);
+	NWO_CONNECT (priv->convert_SF, priv->source);
 
 	return gr_open(rig);
 }
 
 
 /*
+ * graudio, mono
  * sound card source override
  */
 int graudio_open(RIG *rig)
@@ -338,8 +354,27 @@ int graudio_open(RIG *rig)
 
 	/*
 	 * assumes sound card is full duplex!
+	 * mono source
 	 */
-	priv->source =  new VrAudioSource<short>(priv->input_rate);
+	priv->source =  new GrAudioSource<VrComplex>(priv->input_rate, 1,1,AUDIO_SRC);
+
+	return gr_open(rig);
+}
+
+
+/*
+ * graudio I&Q, stereo
+ * sound card source override
+ */
+int graudioiq_open(RIG *rig)
+{
+	struct gnuradio_priv_data *priv = (struct gnuradio_priv_data*)rig->state.priv;
+
+	/*
+	 * assumes sound card is full duplex!
+	 * I&Q source
+	 */
+	priv->source =  new GrAudioSource<VrComplex>(priv->input_rate, 2,1, AUDIO_SRC);
 
 	return gr_open(rig);
 }
@@ -366,13 +401,10 @@ int gr_close(RIG *rig)
   delete priv->m;
   delete priv->sink;
   delete priv->source;
-  if (priv->need_fixer)
-  	delete priv->offset_fixer;
 
   priv->m = NULL;
   priv->sink = NULL;
   priv->source = NULL;
-  priv->need_fixer = 0;
 
   rig_close(priv->tuner);
 
@@ -422,9 +454,14 @@ static int update_freq(RIG *rig, unsigned chan_num, freq_t tuner_freq, freq_t fr
 {
   struct gnuradio_priv_data *priv = (struct gnuradio_priv_data *)rig->state.priv;
   channel_t *chan = &priv->chans[chan_num];
-  struct mod_data *mod = &priv->mods[chan_num];
+  DemodChainCF *mod = priv->mods[chan_num];
   double freq_offset;
 
+  if (chan->mode == RIG_MODE_NONE || !mod) {
+	rig_debug(RIG_DEBUG_TRACE,"No (de)modulator for chan %d\n",chan_num);
+	return RIG_OK;
+  }
+
   /*
    * In case the tuner is not a real tuner
    */
@@ -432,34 +469,19 @@ static int update_freq(RIG *rig, unsigned chan_num, freq_t tuner_freq, freq_t fr
 	  tuner_freq = priv->IF_center_freq;
 
   freq_offset = (double) (freq - tuner_freq);
-  rig_debug(RIG_DEBUG_VERBOSE, "%s: %lld %lld freq_offset=%g\n",
+  rig_debug(RIG_DEBUG_VERBOSE, "%s: tuner:%lld gr:%lld freq_offset=%g\n",
 		  __FUNCTION__, tuner_freq, freq, freq_offset);
 
   if (freq_offset == 0)
 	  return RIG_OK;	/* nothing to do */
 
-  /* TODO: change mode plumbing (don't forget locking!)
-   * workaround?: set_mode(NONE), set_mode(previous)
-   */
   pthread_mutex_lock(&priv->mutex_process);
-  switch (chan->mode) {
-  case RIG_MODE_WFM:
-  case RIG_MODE_FM:
-  case RIG_MODE_USB:
-	  /* not so sure about if setCenter_Freq is the Right thing to do(tm). */
-	mod->chan_filter->setCenterFreq(priv->IF_center_freq + freq_offset);
-	break;
-	/*
-	 * set_freq for SSB mod
-	 * mod->ssb.shifter->set_freq(2*M_PI*freq_offset/(double)priv->input_rate);
-	 */
-  case RIG_MODE_NONE:
-	break;  
-  default:
-	  rig_debug(RIG_DEBUG_WARN, "%s: mode %s unimplemented!\n",
-			  __FUNCTION__, strrmode(chan->mode));
-	  break;
-  }
+
+  /*
+   * not so sure about if setCenter_Freq is the Right thing to do(tm).
+   */
+  mod->setFreq((freq_t)(priv->IF_center_freq + freq_offset));
+
   pthread_mutex_unlock(&priv->mutex_process);
 
   return RIG_OK;
@@ -493,12 +515,18 @@ int gr_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 	  /*
 	   * do not set tuner to freq, but center it
 	   */
-	  ret = rig_set_freq(priv->tuner, RIG_VFO_CURR, freq + GR_MAX_FREQUENCY(priv)/2);
+#if 0
+	  if (GR_MAX_FREQUENCY(priv)/2 < mode_offset)
+		  tuner_freq = freq - mode_offset - resolution;
+	  else
+#endif
+		  tuner_freq = freq - GR_MAX_FREQUENCY(priv)/2;
+	  ret = rig_set_freq(priv->tuner, RIG_VFO_CURR, tuner_freq);
 	  if (ret != RIG_OK)
 		  return ret;
 	  /*
-	   * query freq right back, because wanted freq may not be real freq,
-	   * because of resolution of tuner
+	   * query freq right back, because wanted freq may be rounded
+	   * because of the resolution of the tuner
 	   */
   	  ret = rig_get_freq(priv->tuner, RIG_VFO_CURR, &tuner_freq);
 	  if (ret != RIG_OK)
@@ -543,7 +571,7 @@ int gr_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
   struct gnuradio_priv_data *priv = (struct gnuradio_priv_data *)rig->state.priv;
   int chan_num = vfo2chan_num(rig, vfo);
   channel_t *chan = &priv->chans[chan_num];
-  struct mod_data *mod = &priv->mods[chan_num];
+  DemodChainCF *mod = priv->mods[chan_num];
   char buf[16];
   freq_t tuner_freq;
   int ret = RIG_OK;
@@ -558,10 +586,6 @@ int gr_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
   if (mode == chan->mode && width == chan->width)
 	  return RIG_OK;
 
-  /*
-   * TODO: check if only change in width
-   */
-
 
   ret = rig_get_freq(priv->tuner, RIG_VFO_CURR, &tuner_freq);
   if (ret != RIG_OK)
@@ -572,284 +596,76 @@ int gr_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
   if (priv->tuner_model == RIG_MODEL_DUMMY)
 	  tuner_freq = 0;
 
-  freq_offset = (double) (chan->freq - tuner_freq);
-  rig_debug(RIG_DEBUG_VERBOSE, "%s: freq_offset=%g\n",
-		  __FUNCTION__, freq_offset);
+  freq_offset = (double) (chan->freq - tuner_freq + priv->IF_center_freq);
+  rig_debug(RIG_DEBUG_VERBOSE, "%s: freq_offset=%g tuner_freq=%lld, IFcenter=%ld\n",
+		  __FUNCTION__, freq_offset, tuner_freq, priv->IF_center_freq);
+
+
+
 
   pthread_mutex_lock(&priv->mutex_process);
 
+  /* Same mode, but different width */
+  if (mode != RIG_MODE_NONE && mode == chan->mode) {
+	mod->setWidth(width);
+	chan->width = width;
+	pthread_mutex_unlock(&priv->mutex_process);
+	return RIG_OK;
+  }
+
+  priv->m->stop();
+
   /* TODO: destroy GNUradio connections beforehand if different mode? */
   if (chan->mode != RIG_MODE_NONE && mode != chan->mode) {
 
-  	switch (chan->mode) {
-	  case RIG_MODE_FM:
-	  case RIG_MODE_WFM:
-		delete mod->demod.wfm.demod;
-		delete mod->chan_filter;
-		delete mod->audio_filter;
-		break;
-	  case RIG_MODE_USB:
-		delete mod->audio_filter;
-		delete mod->chan_filter;
-		break;
-	  case RIG_MODE_AM:
-		delete mod->chan_filter;
-		delete mod->audio_filter;
-	  default:
-		break;
-	  }
+	delete mod;
+  	priv->mods[chan_num] = NULL;
+
+	delete priv->m;
+	priv->m = new VrMultiTask ();
   }
 
-  /* TODO: if same mode, but different width, then adapt */
- 
-  priv->m->stop();
-
-  switch (mode) {
-  case RIG_MODE_FM:
-	  {
-  mod->CFIRdecimate = 20;
-  mod->CFIRdecimate2 = 25;
-  mod->RFIRdecimate = 5;
-
-const float channelSpacing = 80e3;
-const float demodBW = 3e3;
-const float TAU = 75e-6;  // 75us in US, 50us in EUR  
-
-const int quadRate = priv->input_rate / mod->CFIRdecimate / mod->CFIRdecimate2;
-const int audioRate = quadRate / mod->RFIRdecimate;
-
-   
-    // 
-    // design first stage selection filter
-    //
-    
-    // width of transition band.
-    //
-    // We make this twice as wide as you would think.  This allows
-    // some aliasing, but it's outside our final bandwidth, determined
-    // by the next filter
-
-    float transition_width = (priv->input_rate / mod->CFIRdecimate - width);
-
-    vector<float> cs1_coeffs =
-      gr_firdes::low_pass (1.0,				// gain
-			   priv->input_rate,			// sampling rate
-			   width / 2,		// low-pass cutoff freq
-			   transition_width,
-			   gr_firdes::WIN_HANN);
-    
-    cerr << "Number of cs1_coeffs: " << cs1_coeffs.size () << endl;
-    
-    //
-    // design second stage channel selection filter
-    //
-    // This is the one that we tune
-    //
-    vector<float> cs2_coeffs =
-      gr_firdes::low_pass (1.0,
-			   priv->input_rate / mod->CFIRdecimate,
-			   width / 2,
-			   (channelSpacing - width) / 2,
-			   gr_firdes::WIN_HANN);
-    
-    cerr << "Number of cs2_coeffs: " << cs2_coeffs.size () << endl;
-    
-    //
-    // design audio filter
-    //
-    vector<float> audio_coeffs =
-      gr_firdes::band_pass (1e3,			// gain
-			    quadRate,			// sampling rate
-			    300,
-			    demodBW,
-			    250,
-			    gr_firdes::WIN_HAMMING);
-    
-    cerr << "Number of audio_coeffs: " << audio_coeffs.size () << endl;
-    
-    //
-    // design demphasis filter
-    //
-    vector<double> fftaps = vector<double>(2);
-    vector<double> fbtaps = vector<double>(2);
-    
-    fftaps[0] = 1 - exp(-1/(TAU*audioRate));
-    fftaps[1] = 0;
-    fbtaps[0] = 0;
-    fbtaps[1] = exp(-1/TAU/priv->input_rate*50);;
-
-    // 
-    // now instantiate the modules
-    //
-
-    // short --> VrComplex
-    mod->chan_filter = 
-    		new GrFreqXlatingFIRfilterSCF (mod->CFIRdecimate,
-						cs1_coeffs,
-						priv->IF_center_freq - freq_offset);
-    
-    // VrComplex --> VrComplex
-    mod->demod.fm.chan2_filter = 
-    		new GrFreqXlatingFIRfilterCCF (mod->CFIRdecimate2, cs2_coeffs, 0);
-   
-
-    // VrComplex --> float
-    mod->demod.fm.demod = new VrQuadratureDemod<float>(1);
-    
-
-    // float --> float
-    mod->audioF_filter = new GrFIRfilterFFF (mod->RFIRdecimate, audio_coeffs);
-    
-    // float --> float
-    mod->demod.fm.deemph =
-      new GrIIRfilter<float,float,double> (1,fftaps,fbtaps); 
-    
-    // float --> short
-    mod->demod.fm.cfs = new GrConvertFS ();
-    
-    //connect the modules together
-    
-	NWO_CONNECT (GR_SOURCE(priv), mod->chan_filter);
-	NWO_CONNECT (mod->chan_filter, mod->demod.fm.chan2_filter);
-	NWO_CONNECT (mod->demod.fm.chan2_filter, mod->demod.fm.demod);
-	NWO_CONNECT (mod->demod.fm.demod, mod->audioF_filter);
-	NWO_CONNECT (mod->audioF_filter, mod->demod.fm.deemph);
-	NWO_CONNECT (mod->demod.fm.deemph, mod->demod.fm.cfs);
-	NWO_CONNECT (mod->demod.fm.cfs, priv->sink);
-
-	break;
-	  }
-
-  case RIG_MODE_WFM:
-	  {
-  mod->CFIRdecimate = 125;
-  mod->RFIRdecimate = 5;
-
-	const int quadRate = priv->input_rate / mod->CFIRdecimate;
-	const int audioRate = quadRate / mod->RFIRdecimate;
-
-	rig_debug(RIG_DEBUG_VERBOSE, "Input Sampling Rate: %d\n", priv->input_rate);
-	rig_debug(RIG_DEBUG_VERBOSE, "Complex FIR decimation factor: %d\n", mod->CFIRdecimate);
-	rig_debug(RIG_DEBUG_VERBOSE, "QuadDemod Sampling Rate: %d\n", quadRate);
-	rig_debug(RIG_DEBUG_VERBOSE, "Real FIR decimation factor: %d\n", mod->RFIRdecimate);
-	rig_debug(RIG_DEBUG_VERBOSE, "Audio Sampling Rate: %d\n", audioRate);
-
- 
-    // build channel filter
-    //
-    // note that the totally bogus transition width is because
-    // we don't have enough mips right now to really do the right thing.
-    // This results in a filter with 83 taps, which is just a few
-    // more than the original 75 in microtune_fm_demo.
-     vector<float> channel_coeffs =
-      gr_firdes::low_pass (1.0,				// gain
-			   priv->input_rate,		// sampling rate
-			   width,			// low-pass cutoff: freq 250e3
-			   8*100e3,			// width of transition band
-			   gr_firdes::WIN_HAMMING);
-
-	rig_debug(RIG_DEBUG_VERBOSE, "Number of channel_coeffs: %d\n", channel_coeffs.size ());
-  
-    // short --> VrComplex
-    mod->chan_filter = 
-	    new GrFreqXlatingFIRfilterSCF(mod->CFIRdecimate, channel_coeffs, 
-			    priv->IF_center_freq - freq_offset);
-
-
-    // float --> short
-    double width_of_transition_band = audioRate / 32;
-    vector<float> audio_coeffs =
-      gr_firdes::low_pass (1.0,				// gain
-			   quadRate,			// sampling rate
-			   audioRate/2 - width_of_transition_band, // low-pass cutoff freq
-			   width_of_transition_band,
-			   gr_firdes::WIN_HAMMING);
-
-	rig_debug(RIG_DEBUG_VERBOSE, "Number of audio_coeffs: %d\n", audio_coeffs.size ());
-	rig_debug(RIG_DEBUG_VERBOSE, "Low-pass cutoff freq: %d\n", audioRate/2 - (int)width_of_transition_band);
-  
-    mod->audio_filter = 
-      new GrFIRfilterFSF(mod->RFIRdecimate, audio_coeffs);
-
-  	mod->demod.wfm.FMdemodGain = 2200;
-	float volume = chan->levels[rig_setting2idx(RIG_LEVEL_AF)].f;
-
-	//
-	// setup Wide FM demodulator chain
-	//
-    // VrComplex --> float
-    mod->demod.wfm.demod =
-      new VrQuadratureDemod<float>(volume * mod->demod.wfm.FMdemodGain);
-
-    //connect the modules together
-
-	NWO_CONNECT (GR_SOURCE(priv), mod->chan_filter);
-	NWO_CONNECT (mod->chan_filter, mod->demod.wfm.demod);
-	NWO_CONNECT (mod->demod.wfm.demod, mod->audio_filter);
-	NWO_CONNECT (mod->audio_filter, priv->sink);
-
-	break;
-	  }
-
-  case RIG_MODE_USB:
-	  {
-	float rf_gain = chan->levels[rig_setting2idx(RIG_LEVEL_RF)].f;
-
-
-#if 0
-	 //SSB mod:
-	mod->ssb.hilb = new GrHilbert<short>(31);	/* what's that 31? */
-	mod->ssb.shifter = new GrSSBMod<short>(2*M_PI*freq_offset/(double)priv->input_rate,
-			rf_gain);
-#endif
-
-	//connect the modules together
-	NWO_CONNECT (GR_SOURCE(priv), mod->chan_filter);
-	NWO_CONNECT (mod->chan_filter, mod->demod.fm.demod);
-	NWO_CONNECT (mod->demod.fm.demod, mod->audio_filter);
-	NWO_CONNECT (mod->audio_filter, priv->sink);
-
-	break;
-	  }
-
-#if 0
-  case RIG_MODE_AM:
-	  {
-	float volume = chan->levels[rig_setting2idx(RIG_LEVEL_AF)].f;
-
-	//
-	// setup Wide FM demodulator chain
-	//
-
-    // VrComplex --> float
-    mod->am.demod =
-      new VrAmplitudeDemod<float>(0.0, 0.05);
-
-    //connect the modules together
-
-	NWO_CONNECT (GR_SOURCE(priv), mod->chan_filter);
-	NWO_CONNECT (mod->chan_filter, mod->am.demod);
-	NWO_CONNECT (mod->am.demod, mod->audio_filter);
-	NWO_CONNECT (mod->audio_filter, priv->sink);
-
-	break;
-	  }
-#endif
-
-  case RIG_MODE_NONE:
+  if (mode == RIG_MODE_NONE) {
 	  /* ez */
-	break;  
+	chan->mode = mode;
+	chan->width = width;
+  	pthread_mutex_unlock(&priv->mutex_process);
+	return RIG_OK;
+  }
 
+  float rf_gain = chan->levels[rig_setting2idx(RIG_LEVEL_RF)].f;
+
+  switch(mode) {
+  case RIG_MODE_LSB:
+	mod = new LSBDemodChainCF(priv->source, priv->sink, mode, width, priv->input_rate, (freq_t)freq_offset);
+	break;
+  case RIG_MODE_USB:
+	mod = new USBDemodChainCF(priv->source, priv->sink, mode, width, priv->input_rate, (freq_t)freq_offset);
+	break;
+  case RIG_MODE_AM:
+	mod = new AMDemodChainCF(priv->source, priv->sink, mode, width, priv->input_rate, (freq_t)freq_offset);
+	break;
+  case RIG_MODE_FM:
+	mod = new FMDemodChainCF(priv->source, priv->sink, mode, width, priv->input_rate, (freq_t)freq_offset);
+	break;
+  case RIG_MODE_WFM:
+	mod = new WFMDemodChainCF(priv->source, priv->sink, mode, width, priv->input_rate, (freq_t)freq_offset);
+	break;
   default:
 	  ret = -RIG_EINVAL;
   }
 
+  priv->mods[chan_num] = mod;
+
+  /* wire up the chain */
+  mod->connect();
+
   priv->m->add (priv->sink);
   priv->m->start();
 
-
   pthread_mutex_unlock(&priv->mutex_process);
 
+
   if (ret == RIG_OK) {
 	chan->mode = mode;
 	chan->width = width;
@@ -901,42 +717,14 @@ int gr_get_vfo(RIG *rig, vfo_t *vfo)
   return RIG_OK;
 }
 
-static int set_rf_gain(RIG *rig, channel_t *chan, struct mod_data *mod, float gain)
-{
-  struct gnuradio_priv_data *priv = (struct gnuradio_priv_data *)rig->state.priv;
-
-  pthread_mutex_lock(&priv->mutex_process);
-  switch (chan->mode) {
-  case RIG_MODE_USB:
-	  {
-	//mod->ssb.shifter->set_gain(gain);
-	break;
-	  }
-  case RIG_MODE_NONE:
-	break;  
-  case RIG_MODE_WFM:
-  case RIG_MODE_FM:
-	  /* chan_filter/VrComplexFIRfilter is missing a setGain method! */
-  default:
-	  rig_debug(RIG_DEBUG_WARN, "%s: mode %s unimplemented!\n",
-			  __FUNCTION__, strrmode(chan->mode));
-	  break;
-  }
-  pthread_mutex_unlock(&priv->mutex_process);
-
-  return RIG_OK;
-}
-
 /*
- * TODO:
- * 	restart machinery after level change (e.g. volume, etc.)
  */
 int gnuradio_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 {
   struct gnuradio_priv_data *priv = (struct gnuradio_priv_data *)rig->state.priv;
   int chan_num = vfo2chan_num(rig, vfo);
   channel_t *chan = &priv->chans[chan_num];
-  struct mod_data *mod = &priv->mods[chan_num];
+  DemodChainCF *mod = priv->mods[chan_num];
   char lstr[32];
   int idx;
   int ret = RIG_OK;
@@ -953,16 +741,21 @@ int gnuradio_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 				  strlevel(level), lstr);
   /* TODO: check val is in range */
 
+  pthread_mutex_lock(&priv->mutex_process);
+
   switch (level) {
   case RIG_LEVEL_RF:
-	ret = set_rf_gain(rig, chan, mod, val.f);
+	  /* line-in level of sound card, etc. */
 	break;
   default:
-	rig_debug(RIG_DEBUG_WARN, "%s: level %s unimplemented!\n",
+	rig_debug(RIG_DEBUG_TRACE, "%s: level %s, try tuner\n",
 			  __FUNCTION__, strlevel(level));
+	ret = rig_set_level(priv->tuner, vfo, level, val);
 	break;
   }
 
+  pthread_mutex_unlock(&priv->mutex_process);
+
   return ret;
 }
 
@@ -1075,68 +868,15 @@ int gnuradio_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
   return ret;
 }
 
-int initrigs_gnuradio(void *be_handle)
+DECLARE_INITRIG_BACKEND(gnuradio)
 {
 	rig_debug(RIG_DEBUG_VERBOSE, "gnuradio: _init called\n");
 
 	rig_register(&gr_caps);
 	rig_register(&mc4020_caps);
 	rig_register(&graudio_caps);
+	rig_register(&graudioiq_caps);
 
 	return RIG_OK;
 }
 
-#if 0
-    VrGUI *guimain = 0;
-    VrGUILayout *horiz = 0;
-    VrGUILayout *vert = 0;
-
-
-    if (use_gui_p){
-      guimain = new VrGUI(argc, argv);
-      horiz = guimain->top->horizontal();
-      vert = horiz->vertical();
-    }
-
-    VrSink<VrComplex> *fft_sink1 = 0;
-    VrSink<float> *fft_sink2 = 0;
-    VrSink<short> *fft_sink3 = 0;
-
-    if (use_gui_p){
-      // sink1 is channel filter output
-      fft_sink1 = new GrFFTSink<VrComplex>(vert, 50, 130, 512);
-
-      // sink2 is fm demod output
-      fft_sink2 = new GrFFTSink<float>(vert, 40, 140, 512);
-
-      // sink3 is audio output
-      fft_sink3 = new GrFFTSink<short>(horiz, 40, 140, 512);
-    }
-
-    if (use_gui_p)
-      NWO_CONNECT (chan_filter, fft_sink1);
-
-    if (use_gui_p)
-      NWO_CONNECT (demod, fft_sink2);
-
-    if (use_gui_p)
-      NWO_CONNECT (audio_filter, fft_sink3);
-
-    VrMultiTask *m = new VrMultiTask ();
-    if (use_gui_p){
-      m->add (fft_sink1);
-      m->add (fft_sink3);
-      m->add (fft_sink2);
-    }
-
-    m->add (final_sink);
-
-        if (use_gui_p)
-      guimain->start ();
-
-    
-    while (1){
-      if (use_gui_p)
-	guimain->processEvents(10 /*ms*/);
-
-#endif
