@@ -2,7 +2,7 @@
  *  Hamlib CI-V backend - description of IC-775 and variations
  *  Copyright (c) 2000-2002 by Stephane Fillod
  *
- *	$Id: ic775.c,v 1.3 2002/11/04 22:40:54 fillods Exp $
+ *	$Id: ic775.c,v 1.5 2003/11/16 17:14:43 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -24,6 +24,8 @@
 #include "config.h"
 #endif
 
+#include <stdlib.h>
+
 #include <hamlib/rig.h>
 #include "icom.h"
 
@@ -49,8 +51,7 @@
 static const struct icom_priv_caps ic775_priv_caps = { 
 		0x46,	/* default address */
 		0,		/* 731 mode */
-		ic737_ts_sc_list,
-		IC775_STR_CAL
+		ic737_ts_sc_list
 };
 
 const struct rig_caps ic775_caps = {
