@@ -2,7 +2,7 @@
  *  Hamlib CI-V backend - main file
  *  Copyright (c) 2000-2003 by Stephane Fillod
  *
- *	$Id: icom.c,v 1.76 2003/04/26 09:54:49 fillods Exp $
+ *	$Id: icom.c,v 1.84 2004/02/02 22:15:17 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
@@ -204,6 +205,7 @@ const struct confparams icom_cfg_params[] = {
  * TODO: sort this list with most frequent rigs first.
  */
 static const struct icom_addr icom_addr_list[] = {
+		{ RIG_MODEL_IC703, 0x68 },
 		{ RIG_MODEL_IC706, 0x48 },
 		{ RIG_MODEL_IC706MKII, 0x4e },
 		{ RIG_MODEL_IC706MKIIG, 0x58 },
@@ -241,10 +243,11 @@ static const struct icom_addr icom_addr_list[] = {
 		{ RIG_MODEL_IC970, 0x2e },
 		{ RIG_MODEL_IC1271, 0x24 },
 		{ RIG_MODEL_IC1275, 0x18 },
+		{ RIG_MODEL_ICR10, 0x52 },
 		{ RIG_MODEL_ICR71, 0x1a },
 		{ RIG_MODEL_ICR72, 0x32 },
 		{ RIG_MODEL_ICR75, 0x5a },
-		{ RIG_MODEL_ICR78, 0x62 },
+		{ RIG_MODEL_IC78, 0x62 },
 		{ RIG_MODEL_ICR7000, 0x08 },
 		{ RIG_MODEL_ICR7100, 0x34 },
 		{ RIG_MODEL_ICR8500, 0x4a },
@@ -252,6 +255,7 @@ static const struct icom_addr icom_addr_list[] = {
 		{ RIG_MODEL_MINISCOUT, 0x94 },
 		{ RIG_MODEL_IC718, 0x5e },
 		{ RIG_MODEL_OS535, 0x80 },
+		{ RIG_MODEL_ICID1, 0x01 },
 		{ RIG_MODEL_NONE, 0 },
 };
 
@@ -297,7 +301,6 @@ int icom_init(RIG *rig)
 
 		priv->re_civ_addr = priv_caps->re_civ_addr;
 		priv->civ_731_mode = priv_caps->civ_731_mode;
-		memcpy(&priv->str_cal, &priv_caps->str_cal, sizeof(cal_table_t));
 
 		return RIG_OK;
 }
@@ -530,6 +533,9 @@ int icom_set_vfo(RIG *rig, vfo_t vfo)
 		rs = &rig->state;
 		priv = (struct icom_priv_data*)rs->priv;
 
+		if (vfo == RIG_VFO_CURR)
+			return RIG_OK;
+
 		switch(vfo) {
 		case RIG_VFO_A: icvfo = S_VFOA; break;
 		case RIG_VFO_B: icvfo = S_VFOB; break;
@@ -761,7 +767,7 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		 *   sort the switch cases with the most frequent first
 		 */
 		switch (level) {
-		case RIG_LEVEL_STRENGTH:
+		case RIG_LEVEL_RAWSTR:
 			lvl_cn = C_RD_SQSM;
 			lvl_sc = S_SML;
 			break;
@@ -888,8 +894,8 @@ int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 
 
 		switch (level) {
-		case RIG_LEVEL_STRENGTH:
-			val->i = rig_raw2val(icom_val, &priv->str_cal);
+		case RIG_LEVEL_RAWSTR:
+			val->i = icom_val;
 			break;
 		case RIG_LEVEL_SQLSTAT:
 			/*
@@ -1256,11 +1262,30 @@ int icom_get_rptr_offs(RIG *rig, vfo_t vfo, shortfreq_t *rptr_offs)
  * Assumes rig!=NULL, rig->state.priv!=NULL,
  * 	icom_set_vfo,icom_set_freq works for this rig
  * FIXME: status
+ *
+ * Assumes also that the current VFO is the rx VFO.
  */
 int icom_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 {
 		int status;
 
+		/* This method works also in memory mode(RIG_VFO_MEM) */
+		if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
+			status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
+			if (status != RIG_OK)
+				return status;
+
+			status = icom_set_freq(rig, RIG_VFO_CURR, tx_freq);
+			if (status != RIG_OK)
+				return status;
+
+			status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
+			if (status != RIG_OK)
+				return status;
+
+			return 0;
+		}
+
 		status = icom_set_vfo(rig, RIG_VFO_B);
 		if (status != RIG_OK)
 				return status;
@@ -1273,7 +1298,6 @@ int icom_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 		if (status != RIG_OK)
 				return status;
 
-
 		return status;
 }
 
@@ -1287,6 +1311,23 @@ int icom_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 {
 		int status;
 
+		/* This method works also in memory mode(RIG_VFO_MEM) */
+		if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
+			status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
+			if (status != RIG_OK)
+				return status;
+
+			status = icom_get_freq(rig, RIG_VFO_CURR, tx_freq);
+			if (status != RIG_OK)
+				return status;
+
+			status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
+			if (status != RIG_OK)
+				return status;
+
+			return 0;
+		}
+
 		status = icom_set_vfo(rig, RIG_VFO_B);
 		if (status != RIG_OK)
 				return status;
@@ -1312,6 +1353,23 @@ int icom_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width
 {
 		int status;
 
+		/* This method works also in memory mode(RIG_VFO_MEM) */
+		if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
+			status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
+			if (status != RIG_OK)
+				return status;
+
+			status = icom_set_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
+			if (status != RIG_OK)
+				return status;
+
+			status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
+			if (status != RIG_OK)
+				return status;
+
+			return 0;
+		}
+
 		status = icom_set_vfo(rig, RIG_VFO_B);
 		if (status != RIG_OK)
 				return status;
@@ -1338,15 +1396,32 @@ int icom_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_wid
 {
 		int status;
 
+		/* This method works also in memory mode(RIG_VFO_MEM) */
+		if (rig_has_vfo_op(rig, RIG_OP_XCHG)) {
+			status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
+			if (status != RIG_OK)
+				return status;
+
+			status = icom_get_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
+			if (status != RIG_OK)
+				return status;
+
+			status = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
+			if (status != RIG_OK)
+				return status;
+
+			return 0;
+		}
+
 		status = icom_set_vfo(rig, RIG_VFO_B);
 		if (status != RIG_OK)
 				return status;
 
-		status |= icom_get_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
+		status = icom_get_mode(rig, RIG_VFO_CURR, tx_mode, tx_width);
 		if (status != RIG_OK)
 				return status;
 
-		status |= icom_set_vfo(rig, RIG_VFO_A);
+		status = icom_set_vfo(rig, RIG_VFO_A);
 		if (status != RIG_OK)
 				return status;
 
@@ -1377,7 +1452,7 @@ int icom_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
 			split_sc = S_SPLT_ON;
 			break;
 		default:
-			rig_debug(RIG_DEBUG_ERR,"Unsupported split %d", split);
+			rig_debug(RIG_DEBUG_ERR,"%s: Unsupported split %d", __FUNCTION__, split);
 			return -RIG_EINVAL;
 		}
 
@@ -1395,9 +1470,8 @@ int icom_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
 		return RIG_OK;
 }
 
-
 /*
- * icom_get_split
+ * icom_get_split_vfo
  * Assumes rig!=NULL, rig->state.priv!=NULL, split!=NULL
  */
 int icom_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
@@ -1420,8 +1494,8 @@ int icom_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
 		 */
 		split_len--;
 		if (split_len != 1) {
-				rig_debug(RIG_DEBUG_ERR,"icom_get_split: wrong frame len=%d\n",
-								split_len);
+				rig_debug(RIG_DEBUG_ERR,"%s: wrong frame len=%d\n",
+						__FUNCTION__, split_len);
 				return -RIG_ERJCTED;
 		}
 
@@ -1440,6 +1514,36 @@ int icom_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
 		return RIG_OK;
 }
 
+
+/*
+ * icom_mem_get_split_vfo
+ * Assumes rig!=NULL, rig->state.priv!=NULL, split!=NULL
+ */
+int icom_mem_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
+{
+		int retval;
+
+		/* this hacks works only when in memory mode
+		 * I have no clue how to detect split in regular VFO mode
+		 */
+		if (rig->state.current_vfo != RIG_VFO_MEM)
+			return -RIG_ENAVAIL;
+
+		retval = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
+		if (retval == RIG_OK) {
+			*split = RIG_SPLIT_ON;
+			/* get it back to normal */
+			retval = icom_vfo_op(rig, vfo, RIG_OP_XCHG);
+		} else if (retval == -RIG_ERJCTED) {
+			*split = RIG_SPLIT_OFF;
+		} else {
+			/* this is really an error! */
+			return retval;
+		}
+
+		return RIG_OK;
+}
+
 /*
  * icom_set_ts
  * Assumes rig!=NULL, rig->caps->priv!=NULL
@@ -1914,124 +2018,6 @@ int icom_get_ctcss_sql(RIG *rig, vfo_t vfo, unsigned int *tone)
 		return RIG_OK;
 }
 
-/*
- * icr75_set_channel
- * Assumes rig!=NULL, rig->state.priv!=NULL, chan!=NULL
- * TODO: still a WIP --SF
- */
-int icr75_set_channel(RIG *rig, const channel_t *chan)
-{
-		struct icom_priv_data *priv;
-		struct rig_state *rs;
-		unsigned char chanbuf[MAXFRAMELEN], ackbuf[MAXFRAMELEN];
-		int chan_len, freq_len, ack_len, retval;
-		unsigned char icmode;
-		signed char icmode_ext;
-		int err;
-
-		rs = &rig->state;
-		priv = (struct icom_priv_data*)rs->priv;
-
-		to_bcd_be(chanbuf,chan->channel_num,4);
-
-		chanbuf[2] = S_MEM_CNTNT_SLCT;
-
-		freq_len = priv->civ_731_mode ? 4:5;
-		/*
-		 * to_bcd requires nibble len
-		 */
-		to_bcd(chanbuf+3, chan->freq, freq_len*2);
-
-		chan_len = 3+freq_len+1;
-
-		err = rig2icom_mode(rig, chan->mode, RIG_PASSBAND_NORMAL,
-						&icmode, &icmode_ext);/* FIXME */
-		if (err != RIG_OK)
-				return err;
-
-		chanbuf[chan_len++] = icmode;
-		chanbuf[chan_len++] = icmode_ext;	/* FIXME */
-
-		to_bcd_be(chanbuf+chan_len++,
-						chan->levels[rig_setting2idx(RIG_LEVEL_ATT)].i, 2);
-		to_bcd_be(chanbuf+chan_len++,
-						chan->levels[rig_setting2idx(RIG_LEVEL_PREAMP)].i, 2);
-
-		to_bcd_be(chanbuf+chan_len++,chan->ant,2);
-		memset(chanbuf+chan_len, 0, 8);
-		strncpy(chanbuf+chan_len, chan->channel_desc, 8);
-		chan_len += 8;
-
-		retval = icom_transaction (rig, C_CTL_MEM, S_MEM_CNTNT,
-						chanbuf, chan_len, ackbuf, &ack_len);
-		if (retval != RIG_OK)
-				return retval;
-
-		if (ack_len != 1 || ackbuf[0] != ACK) {
-				rig_debug(RIG_DEBUG_ERR,"icom_set_channel: ack NG (%#.2x), "
-								"len=%d\n", ackbuf[0],ack_len);
-				return -RIG_ERJCTED;
-		}
-
-		return RIG_OK;
-}
-
-/*
- * icr75_get_channel
- * Assumes rig!=NULL, rig->state.priv!=NULL, chan!=NULL
- * TODO: still a WIP --SF
- */
-int icr75_get_channel(RIG *rig, channel_t *chan)
-{
-		struct icom_priv_data *priv;
-		struct rig_state *rs;
-		unsigned char chanbuf[24];
-		int chan_len, freq_len, retval;
-		pbwidth_t width; 	/* FIXME */
-
-		rs = &rig->state;
-		priv = (struct icom_priv_data*)rs->priv;
-
-		to_bcd_be(chanbuf,chan->channel_num,4);
-		chan_len = 2;
-
-		freq_len = priv->civ_731_mode ? 4:5;
-
-		retval = icom_transaction (rig, C_CTL_MEM, S_MEM_CNTNT,
-						chanbuf, chan_len, chanbuf, &chan_len);
-		if (retval != RIG_OK)
-				return retval;
-
-		/*
-		 * freqbuf should contain Cn,Data area
-		 */
-		chan_len--;
-		if (freq_len != freq_len+16) {
-				rig_debug(RIG_DEBUG_ERR,"icr75_get_channel: wrong frame len=%d\n",
-								chan_len);
-				return -RIG_ERJCTED;
-		}
-
-		/*
-		 * from_bcd requires nibble len
-		 */
-		chan->freq = from_bcd(chanbuf+4, freq_len*2);
-
-		chan_len = 4+freq_len+1;
-
-		icom2rig_mode(rig, chanbuf[chan_len], chanbuf[chan_len+1],
-						&chan->mode, &width);
-		chan_len += 2;
-		chan->levels[rig_setting2idx(RIG_LEVEL_ATT)].i =
-				from_bcd_be(chanbuf+chan_len++,2);
-		chan->levels[rig_setting2idx(RIG_LEVEL_PREAMP)].i =
-				from_bcd_be(chanbuf+chan_len++,2);
-		chan->ant = from_bcd_be(chanbuf+chan_len++,2);
-		strncpy(chan->channel_desc, chanbuf+chan_len, 8);
-
-		return RIG_OK;
-}
-
 
 /*
  * icom_set_powerstat
@@ -2147,6 +2133,70 @@ int icom_set_bank(RIG *rig, vfo_t vfo, int bank)
 		return RIG_OK;
 }
 
+/*
+ * icom_set_ant
+ * Assumes rig!=NULL, rig->state.priv!=NULL
+ */
+int icom_set_ant(RIG * rig, vfo_t vfo, ant_t ant)
+{
+		struct icom_priv_data *priv;
+		struct rig_state *rs;
+		unsigned char antarg;
+		unsigned char ackbuf[MAXFRAMELEN];
+		int ack_len, retval, i_ant;
+
+		rs = &rig->state;
+		priv = (struct icom_priv_data*)rs->priv;
+
+		/*
+		 * FIXME: IC-756*, IC-746*
+		 */
+		i_ant = ant == RIG_ANT_1 ? 0: 1;
+
+		antarg = 0;
+		retval = icom_transaction (rig, C_CTL_ANT, i_ant,
+						&antarg, 1, ackbuf, &ack_len);
+		if (retval != RIG_OK)
+				return retval;
+
+		if (ack_len != 1 || ackbuf[0] != ACK) {
+				rig_debug(RIG_DEBUG_ERR,"icom_set_ant: ack NG (%#.2x), "
+								"len=%d\n", ackbuf[0], ack_len);
+				return -RIG_ERJCTED;
+		}
+
+		return RIG_OK;
+}
+
+/*
+ * icom_get_ant
+ * Assumes rig!=NULL, rig->state.priv!=NULL
+ */
+int icom_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
+{
+		unsigned char ackbuf[MAXFRAMELEN];
+		int ack_len, retval;
+
+		retval = icom_transaction(rig, C_CTL_ANT, -1, NULL, 0,
+						ackbuf, &ack_len);
+		if (retval != RIG_OK)
+				return retval;
+
+		if (ack_len != 1 || ackbuf[0] != ACK) {
+				rig_debug(RIG_DEBUG_ERR,"icom_get_ant: ack NG (%#.2x), "
+								"len=%d\n", ackbuf[0],ack_len);
+				return -RIG_ERJCTED;
+		}
+
+		/*
+		 * FIXME: IC-756*, IC-746*
+		 */
+		*ant = ackbuf[1] == 0 ? RIG_ANT_1 : RIG_ANT_2;
+
+		return RIG_OK;
+}
+
+
 /*
  * icom_vfo_op, Mem/VFO operation
  * Assumes rig!=NULL, rig->state.priv!=NULL
@@ -2166,16 +2216,6 @@ int icom_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 		mv_len = 0;
 
 		switch(op) {
-#if 0
-			case RIG_MVOP_VFO_MODE:
-				mv_cn = C_SET_VFO;
-				mv_sc = -1;
-				break;
-			case RIG_MVOP_MEM_MODE:
-				mv_cn = C_SET_MEM;
-				mv_sc = -1;
-				break;
-#endif
 			case RIG_OP_CPY:
 				mv_cn = C_SET_VFO;
 				mv_sc = S_BTOA;
@@ -2478,6 +2518,55 @@ DECLARE_PROBERIG_BACKEND(icom)
 							"with CI-V ID %#x, please report to Hamlib "
 							"developers.\n", civ_id);
 		}
+
+		/*
+		 * Try to identify OptoScan
+		 */
+		for (civ_addr=0x80; civ_addr<=0x8f; civ_addr++) {
+
+			frm_len = make_cmd_frame(buf, civ_addr, C_CTL_MISC, S_OPTO_RDID,
+							NULL, 0);
+
+			serial_flush(port);
+			write_block(port, buf, frm_len);
+
+			/* read out the bytes we just sent
+		 	* TODO: check this is what we expect
+		 	*/
+			frm_len = read_icom_frame(port, buf);
+
+			/* this is the reply */
+			frm_len = read_icom_frame(port, buf);
+
+			/* timeout.. nobody's there */
+			if (frm_len <= 0)
+					continue;
+
+			/* wrong protocol? */
+			if (frm_len != 7 || buf[4] != C_CTL_MISC || buf[5] != S_OPTO_RDID) {
+				continue;
+			}
+
+			rig_debug(RIG_DEBUG_VERBOSE, "%s, found OptoScan%c%c%c, software version %d.%d, "
+					"interface version %d.%d, at %#x\n",
+					__FUNCTION__,
+					buf[2], buf[3], buf[4],
+					buf[5] >> 4, buf[5] & 0xf,
+					buf[6] >> 4, buf[6] & 0xf,
+					civ_addr);
+
+			if (buf[6] == '5' && buf[7] == '3' && buf[8] == '5')
+				model = RIG_MODEL_OS535;
+			else if (buf[6] == '4' && buf[7] == '5' && buf[8] == '6')
+				model = RIG_MODEL_OS456;
+			else
+				continue;
+
+			if (cfunc)
+				(*cfunc)(port, model, data);
+			break;
+		}
+
 		close(port->fd);
 
 		/*
@@ -2498,27 +2587,43 @@ DECLARE_INITRIG_BACKEND(icom)
 {
 	rig_debug(RIG_DEBUG_VERBOSE, "icom: _init called\n");
 
+	rig_register(&ic703_caps);
 	rig_register(&ic706_caps);
 	rig_register(&ic706mkii_caps);
 	rig_register(&ic706mkiig_caps);
 	rig_register(&ic718_caps);
 	rig_register(&ic725_caps);
+	rig_register(&ic726_caps);
 	rig_register(&ic735_caps);
 	rig_register(&ic736_caps);
 	rig_register(&ic737_caps);
+	rig_register(&ic746_caps);
+	rig_register(&ic746pro_caps);
 	rig_register(&ic775_caps);
 	rig_register(&ic756_caps);
 	rig_register(&ic756pro_caps);
 	rig_register(&ic756pro2_caps);
+	rig_register(&ic765_caps);
+	rig_register(&ic781_caps);
+	rig_register(&ic707_caps);
+	rig_register(&ic728_caps);
 
 	rig_register(&ic821h_caps);
 	rig_register(&ic910_caps);
 	rig_register(&ic970_caps);
 
+        rig_register(&icr10_caps);
+        rig_register(&icr71_caps);
+        rig_register(&icr72_caps);
+        rig_register(&icr75_caps);
         rig_register(&icr7000_caps);
+        rig_register(&icr7100_caps);
 	rig_register(&icr8500_caps);
+	rig_register(&icr9000_caps);
 
+	rig_register(&ic271_caps);
 	rig_register(&ic275_caps);
+	rig_register(&ic471_caps);
 	rig_register(&ic475_caps);
 
 	rig_register(&os535_caps);
@@ -2526,6 +2631,8 @@ DECLARE_INITRIG_BACKEND(icom)
 
 	rig_register(&omnivip_caps);
 
+	rig_register(&id1_caps);
+
 	return RIG_OK;
 }
 
