@@ -2,7 +2,7 @@
  *  Hamlib CI-V backend - main header
  *  Copyright (c) 2000-2003 by Stephane Fillod
  *
- *	$Id: icom.h,v 1.56 2003/04/16 22:30:40 fillods Exp $
+ *	$Id: icom.h,v 1.64 2004/02/02 22:15:17 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -56,12 +56,10 @@
 	.freq = 1,	\
 	.mode = 1,	\
 	.width = 1,	\
-	.rptr_shift = 1,	\
+	.tx_freq = 1,	\
+	.tx_mode = 1,	\
+	.tx_width = 1,	\
 	.rptr_offs = 1,	\
-	.ctcss_tone = 1,	\
-	.ctcss_sql = 1,	\
-	.dcs_code = 1,	\
-	.dcs_sql = 1,	\
 } 
 
 struct ts_sc_list {
@@ -69,18 +67,33 @@ struct ts_sc_list {
 	unsigned char sc;	/* sub command */
 };
 
+/**
+ * \brief Pipelined tuning state data structure.
+ */
+typedef struct rig_pltstate {
+  freq_t freq;
+  freq_t next_freq;
+  rmode_t mode;
+  rmode_t next_mode;
+  pbwidth_t width;
+  pbwidth_t next_width;
+  struct timeval timer_start;
+  struct timeval timer_current;
+  int usleep_time; /* dependent on radio module & serial data rate */
+} pltstate_t;
+
 struct icom_priv_caps {
 	unsigned char re_civ_addr;	/* the remote dlft equipment's CI-V address*/
 	int civ_731_mode; /* Off: freqs on 10 digits, On: freqs on 8 digits */
 	const struct ts_sc_list *ts_sc_list;
-	cal_table_t str_cal;
+	int settle_time; /*!< Receiver settle time, in ms */
 };
 
 
 struct icom_priv_data {
 	unsigned char re_civ_addr;	/* the remote equipment's CI-V address*/
 	int civ_731_mode; /* Off: freqs on 10 digits, On: freqs on 8 digits */
-	cal_table_t str_cal;
+	pltstate_t *pltstate;	/* only on optoscan */
 };
 
 extern const struct ts_sc_list r8500_ts_sc_list[];
@@ -112,6 +125,7 @@ int icom_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_t tx_width
 int icom_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth_t *tx_width);
 int icom_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo);
 int icom_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo);
+int icom_mem_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo);
 int icom_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts);
 int icom_get_ts(RIG *rig, vfo_t vfo, shortfreq_t *ts);
 int icom_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt);
@@ -129,48 +143,57 @@ int icom_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val);
 int icom_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val);
 int icom_set_func(RIG *rig, vfo_t vfo, setting_t func, int status);
 int icom_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status);
-int icr75_set_channel(RIG *rig, const channel_t *chan);
-int icr75_get_channel(RIG *rig, channel_t *chan);
 int icom_set_conf(RIG *rig, token_t token, const char *val);
 int icom_get_conf(RIG *rig, token_t token, char *val);
 int icom_set_powerstat(RIG *rig, powerstat_t status);
 int icom_get_powerstat(RIG *rig, powerstat_t *status);
+int icom_set_ant(RIG * rig, vfo_t vfo, ant_t ant);
+int icom_get_ant(RIG * rig, vfo_t vfo, ant_t *ant);
 int icom_decode_event(RIG *rig);
 
-int optoscan_open(RIG *rig);
-int optoscan_close(RIG *rig);
-const char* optoscan_get_info(RIG *rig);
-int optoscan_get_ctcss_tone(RIG *rig, vfo_t vfo, tone_t *tone);
-int optoscan_get_dcs_code(RIG * rig, vfo_t vfo, tone_t *code);
-int optoscan_recv_dtmf(RIG *rig, vfo_t vfo, char *digits, int *length);
-
 extern const struct confparams icom_cfg_params[];
 
+extern const struct rig_caps ic703_caps;
 extern const struct rig_caps ic706_caps;
 extern const struct rig_caps ic706mkii_caps;
 extern const struct rig_caps ic706mkiig_caps;
+extern const struct rig_caps ic707_caps;
 extern const struct rig_caps ic718_caps;
 extern const struct rig_caps ic725_caps;
+extern const struct rig_caps ic726_caps;
+extern const struct rig_caps ic728_caps;
 extern const struct rig_caps ic735_caps;
 extern const struct rig_caps ic736_caps;
 extern const struct rig_caps ic737_caps;
+extern const struct rig_caps ic746_caps;
+extern const struct rig_caps ic746pro_caps;
 extern const struct rig_caps ic756_caps;
 extern const struct rig_caps ic756pro_caps;
 extern const struct rig_caps ic756pro2_caps;
+extern const struct rig_caps ic765_caps;
 extern const struct rig_caps ic775_caps;
+extern const struct rig_caps ic781_caps;
 extern const struct rig_caps ic821h_caps;
 extern const struct rig_caps ic910_caps;
 extern const struct rig_caps ic970_caps;
+extern const struct rig_caps icr10_caps;
+extern const struct rig_caps icr71_caps;
+extern const struct rig_caps icr72_caps;
+extern const struct rig_caps icr75_caps;
 extern const struct rig_caps icr7000_caps;
+extern const struct rig_caps icr7100_caps;
 extern const struct rig_caps icr8500_caps;
+extern const struct rig_caps icr9000_caps;
+extern const struct rig_caps ic271_caps;
 extern const struct rig_caps ic275_caps;
+extern const struct rig_caps ic471_caps;
 extern const struct rig_caps ic475_caps;
 
+extern const struct rig_caps omnivip_caps;
+
 extern const struct rig_caps os456_caps;
 extern const struct rig_caps os535_caps;
 
-extern const struct rig_caps omnivip_caps;
-
+extern const struct rig_caps id1_caps;
 
 #endif /* _ICOM_H */
-
