@@ -2,7 +2,7 @@
  *  Hamlib Interface - API header
  *  Copyright (c) 2000-2003 by Stephane Fillod and Frank Singleton
  *
- *	$Id: rig.h,v 1.80 2003/04/27 22:15:04 fillods Exp $
+ *	$Id: rig.h,v 1.92 2004/02/09 22:07:21 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -24,13 +24,10 @@
 #define _RIG_H 1
 
 #include <hamlib/riglist.h>	/* list in another file to not mess up w/ this one */
-#include <stdio.h>		/* required for FILE definition */
 #include <sys/time.h>		/* required for struct timeval */
 
 #if defined(__CYGWIN__) || defined(_WIN32)
 #include <windows.h>		/* HANDLE definition */
-#else
-#include <sys/socket.h>		/* required for struct sockaddr */
 #endif
 
 /*! \file rig.h
@@ -95,6 +92,9 @@ enum rig_errcode_e {
 	RIG_ENTARGET,		/*<! VFO not targetable */
 	RIG_BUSERROR,		/*<! Error talking on the bus */
 	RIG_BUSBUSY,		/*<! Collision on the bus */
+	RIG_EARG,		/*<! NULL RIG handle or any invalid pointer parameter in get arg */
+	RIG_EVFO,		/*<! Invalid VFO */
+	RIG_EDOM		/*<! Argument out of domain of func */
 };
 
 /**
@@ -131,9 +131,10 @@ typedef struct rig RIG;
 #define FRQRANGESIZ 30
 #define MAXCHANDESC 30		/* describe channel eg: "WWV 5Mhz" */
 #define TSLSTSIZ 20		/* max tuning step list size, zero ended */
-#define FLTLSTSIZ 16		/* max mode/filter list size, zero ended */
+#define FLTLSTSIZ 42		/* max mode/filter list size, zero ended */
 #define MAXDBLSTSIZ 8		/* max preamp/att levels supported, zero ended */
 #define CHANLSTSIZ 16		/* max mem_list size, zero ended */
+#define MAX_CAL_LENGTH 32	/* max calibration plots in cal_table_t */
 
 
 /**
@@ -160,7 +161,7 @@ typedef enum rig_port_e {
   RIG_PORT_DTMF,		/*!< DTMF protocol bridge via another rig, eg. Kenwood Sky Cmd System */
   RIG_PORT_ULTRA,		/*!< IrDA Ultra protocol! */
   RIG_PORT_RPC,			/*!< RPC wrapper */
-  RIG_PORT_PARALLEL,		/*!< Parallel port */
+  RIG_PORT_PARALLEL		/*!< Parallel port */
 } rig_port_t;
 
 /**
@@ -204,7 +205,7 @@ typedef enum {
 	RIG_FLAG_APRS =		(1<<8),		/*!< has APRS */
 	RIG_FLAG_TNC =		(1<<9),		/*!< has TNC */
 	RIG_FLAG_DXCLUSTER =	(1<<10),	/*!< has DXCluster */
-	RIG_FLAG_TUNER =	(1<<11),	/*!< dumb tuner */
+	RIG_FLAG_TUNER =	(1<<11) 	/*!< dumb tuner */
 } rig_type_t;
 
 #define RIG_FLAG_TRANSCEIVER (RIG_FLAG_RECEIVER|RIG_FLAG_TRANSMITTER)
@@ -248,16 +249,15 @@ typedef enum {
  */
 typedef enum {
   RIG_SPLIT_OFF = 0,		/*!< Split mode disabled */
-  RIG_SPLIT_ON,			/*!< Split mode enabled */
+  RIG_SPLIT_ON			/*!< Split mode enabled */
 } split_t;
 
 /**
  * \brief Frequency type,
- * Frequency type unit in Hz, able to hold SHF frequencies (64bits).
- * This type holds an integer number of Hertz.
- * Hamlib has no support yet for fractional number of Hertz.
+ * Frequency type unit in Hz, able to hold SHF frequencies.
  */
-typedef signed long long freq_t;
+typedef double freq_t;
+#define FREQFMT "lf"
 
 /**
  * \brief Short frequency type
@@ -270,6 +270,11 @@ typedef signed long shortfreq_t;
 #define MHz(f)	((freq_t)((f)*(freq_t)1000000))
 #define GHz(f)	((freq_t)((f)*(freq_t)1000000000))
 
+#define s_Hz(f) 	((shortfreq_t)(f))
+#define s_kHz(f)	((shortfreq_t)((f)*(shortfreq_t)1000))
+#define s_MHz(f)	((shortfreq_t)((f)*(shortfreq_t)1000000))
+#define s_GHz(f)	((shortfreq_t)((f)*(shortfreq_t)1000000000))
+
 #define RIG_FREQ_NONE Hz(0)
 
 
@@ -332,7 +337,7 @@ typedef int vfo_t;
 #define RIG_TARGETABLE_ALL  0xffffffffU
 
 
-#define RIG_PASSBAND_NORMAL Hz(0)
+#define RIG_PASSBAND_NORMAL s_Hz(0)
 /**
  * \brief Passband width, in Hz
  * \sa rig_passband_normal, rig_passband_narrow, rig_passband_wide
@@ -357,7 +362,8 @@ typedef enum {
   RIG_DCD_RIG,			/*!< Rig has DCD status support, i.e. rig has get_dcd cap */
   RIG_DCD_SERIAL_DSR,		/*!< DCD status from serial DSR signal */
   RIG_DCD_SERIAL_CTS,		/*!< DCD status from serial CTS signal */
-  RIG_DCD_PARALLEL,		/*!< DCD status from parallel port pin */
+  RIG_DCD_SERIAL_CAR,		/*!< DCD status from serial CD signal */
+  RIG_DCD_PARALLEL		/*!< DCD status from parallel port pin */
 } dcd_type_t;
 
 
@@ -387,7 +393,7 @@ typedef enum {
 typedef enum {
   RIG_POWER_OFF =	0,		/*!< Power off */
   RIG_POWER_ON =	(1<<0),		/*!< Power on */
-  RIG_POWER_STANDBY =	(1<<1),		/*!< Standby */
+  RIG_POWER_STANDBY =	(1<<1)		/*!< Standby */
 } powerstat_t;
 
 /**
@@ -398,7 +404,7 @@ typedef enum {
   RIG_RESET_SOFT =	(1<<0),		/*!< Software reset */
   RIG_RESET_VFO =	(1<<1),		/*!< VFO reset */
   RIG_RESET_MCALL =	(1<<2),		/*!< Memory clear */
-  RIG_RESET_MASTER =	(1<<3),		/*!< Master reset */
+  RIG_RESET_MASTER =	(1<<3)		/*!< Master reset */
 } reset_t;
 
 
@@ -427,6 +433,7 @@ typedef enum {
 	RIG_OP_LEFT =		(1<<9),	/*!< LEFT */
 	RIG_OP_RIGHT =		(1<<10),/*!< RIGHT */
 	RIG_OP_TUNE =		(1<<11),/*!< Start tune */
+	RIG_OP_TOGGLE =		(1<<12) /*!< Toggle VFOA and VFOB */
 } vfo_op_t;
 
 
@@ -442,6 +449,7 @@ typedef enum {
 	RIG_SCAN_PROG =		(1<<3),	/*!< Programmed(edge) scan */
 	RIG_SCAN_DELTA =	(1<<4),	/*!< delta-f scan */
 	RIG_SCAN_VFO =		(1<<5),	/*!< most basic scan */
+	RIG_SCAN_PLT =		(1<<6)  /*!< Scan using pipelined tuning */
 } scan_t;
 
 /**
@@ -458,7 +466,7 @@ enum rig_conf_e {
 	RIG_CONF_STRING,	/*!<	String type */
 	RIG_CONF_COMBO,		/*!<	Combo type */
 	RIG_CONF_NUMERIC,	/*!<	Numeric type (integer or real) */
-	RIG_CONF_CHECKBUTTON,	/*!<	on/off type */
+	RIG_CONF_CHECKBUTTON	/*!<	on/off type */
 };
 
 #define RIG_COMBO_MAX	8
@@ -495,7 +503,7 @@ typedef enum {
 	RIG_ANN_RXMODE = (1<<1),	/*!< Announce receive mode */
 	RIG_ANN_CW = (1<<2),		/*!< CW */
 	RIG_ANN_ENG = (1<<3),		/*!< English */
-	RIG_ANN_JAP = (1<<4),		/*!< Japan */
+	RIG_ANN_JAP = (1<<4)		/*!< Japan */
 } ann_t;
 
 
@@ -515,7 +523,7 @@ enum agc_level_e {
 	RIG_AGC_SUPERFAST,
 	RIG_AGC_FAST,
 	RIG_AGC_SLOW,
-	RIG_AGC_USER,		/*!< user selectable */
+	RIG_AGC_USER		/*!< user selectable */
 };
 
 /**
@@ -527,7 +535,7 @@ enum meter_level_e {
   RIG_METER_COMP =	(1<<1),		/*< Compression level */
   RIG_METER_ALC =	(1<<2),		/*< ALC */
   RIG_METER_IC =	(1<<3),		/*< IC */
-  RIG_METER_DB =	(1<<4),		/*< DB */
+  RIG_METER_DB =	(1<<4)		/*< DB */
 };
 
 /**
@@ -569,17 +577,19 @@ enum rig_level_e {
 	RIG_LEVEL_VOXGAIN =	(1<<21),/*!< VOX gain level, arg float [0.0 .. 1.0] */
 	RIG_LEVEL_VOXDELAY =  RIG_LEVEL_VOX,	/*!< VOX delay, arg int (tenth of seconds) */
 	RIG_LEVEL_ANTIVOX =	(1<<22),/*!< anti-VOX level, arg float [0.0 .. 1.0] */
+	//RIG_LEVEL_LINEOUT =	(1<<23),/*!< Lineout Volume, arg float [0.0 .. 1.0] */
 
 		/*!< These ones are not settable */
+	RIG_LEVEL_RAWSTR =	(1<<26),/*!< Raw (A/D) value for signal strength, specific to each rig, arg int */
 	RIG_LEVEL_SQLSTAT =	(1<<27),/*!< SQL status, arg int (open=1/closed=0). Deprecated, use get_dcd instead */
 	RIG_LEVEL_SWR =		(1<<28),/*!< SWR, arg float */
 	RIG_LEVEL_ALC =		(1<<29),/*!< ALC, arg float */
-	RIG_LEVEL_STRENGTH =	(1<<30),/*!< Signal strength, arg int (dB) */
+	RIG_LEVEL_STRENGTH =	(1<<30) /*!< Effective (calibrated) signal strength relative to S9, arg int (dB) */
 };
 
 #define RIG_LEVEL_FLOAT_LIST (RIG_LEVEL_AF|RIG_LEVEL_RF|RIG_LEVEL_SQL|RIG_LEVEL_APF|RIG_LEVEL_NR|RIG_LEVEL_PBT_IN|RIG_LEVEL_PBT_OUT|RIG_LEVEL_RFPOWER|RIG_LEVEL_MICGAIN|RIG_LEVEL_COMP|RIG_LEVEL_BALANCE|RIG_LEVEL_SWR|RIG_LEVEL_ALC|RIG_LEVEL_VOXGAIN|RIG_LEVEL_ANTIVOX)
 
-#define RIG_LEVEL_READONLY_LIST (RIG_LEVEL_SQLSTAT|RIG_LEVEL_SWR|RIG_LEVEL_ALC|RIG_LEVEL_STRENGTH)
+#define RIG_LEVEL_READONLY_LIST (RIG_LEVEL_SQLSTAT|RIG_LEVEL_SWR|RIG_LEVEL_ALC|RIG_LEVEL_STRENGTH|RIG_LEVEL_RAWSTR)
 
 #define RIG_LEVEL_IS_FLOAT(l) ((l)&RIG_LEVEL_FLOAT_LIST)
 #define RIG_LEVEL_SET(l) ((l)&~RIG_LEVEL_READONLY_LIST)
@@ -597,7 +607,7 @@ enum rig_parm_e {
 	RIG_PARM_BEEP =		(1<<4),	/*!< Beep on keypressed, int (0,1) */
 	RIG_PARM_TIME =		(1<<5),	/*!< hh:mm:ss, int in seconds from 00:00:00 */
 	RIG_PARM_BAT =		(1<<6),	/*!< battery level, float [0.0..1.0] */
-	RIG_PARM_KEYLIGHT =	(1<<7), /*!< Button backlight, on/off */
+	RIG_PARM_KEYLIGHT =	(1<<7)  /*!< Button backlight, on/off */
 };
 
 #define RIG_PARM_FLOAT_LIST (RIG_PARM_BACKLIGHT|RIG_PARM_BAT)
@@ -611,9 +621,9 @@ enum rig_parm_e {
  * \brief Setting
  *
  * This can be a func, a level or a parm.
- * Each bit designate one of them, let's hope 64 bits will be enough.
+ * Each bit designate one of them.
  */
-typedef unsigned long long setting_t;
+typedef unsigned long setting_t;
 
 /*
  * tranceive mode, ie. the rig notify the host of any event,
@@ -659,7 +669,7 @@ enum rig_func_e {
 	RIG_FUNC_SCOPE =  	(1<<27),/*!< Simple bandscope ON/OFF */
 	RIG_FUNC_RESUME =	(1<<28),/*!< Scan auto-resume */
 	RIG_FUNC_TBURST =	(1<<29),/*!< 1750 Hz tone burst */
-	RIG_FUNC_TUNER =	(1<<30),/*!< Enable automatic tuner */
+	RIG_FUNC_TUNER =	(1<<30) /*!< Enable automatic tuner */
 };
 
 /*
@@ -685,6 +695,10 @@ typedef enum {
 	RIG_MODE_WFM =   	(1<<6),	/*!< broadcast wide FM */
 	RIG_MODE_CWR =   	(1<<7),	/*!< CW reverse sideband */
 	RIG_MODE_RTTYR =	(1<<8),	/*!< RTTY reverse sideband */
+	RIG_MODE_AMS =    	(1<<9),	/*!< Amplitude Modulation Synchronous */
+	RIG_MODE_PKTLSB =       (1<<10),/*!< Packet/Digital LSB mode (dedicated port) */
+	RIG_MODE_PKTUSB =       (1<<11),/*!< Packet/Digital USB mode (dedicated port) */
+	RIG_MODE_PKTFM =        (1<<12) /*!< Packet/Digital FM mode (dedicated port) */
 } rmode_t;
 
 /** \brief macro for backends, no to be used by rig_set_mode et al. */
@@ -926,8 +940,51 @@ struct chan_list {
 /** \brief chan_t type */
 typedef struct chan_list chan_t;
 
+/**
+ * \brief level/parm granularity definition
+ *
+ * The granularity is undefined if min=0, max=0, and step=0.
+ *
+ * For float settings, if min.f=0 and max.f=0 (and step.f!=0),
+ * max.f is assumed to be actually equal to 1.0.
+ *
+ * If step=0 (and min and/or max are not null), then this means step
+ * can have maximum resolution, depending on type (int or float).
+ */
+struct gran {
+	value_t min;		/*!< Minimum value */
+	value_t max;		/*!< Maximum value */
+	value_t step;		/*!< Step */
+};
+
 /** \brief gran_t type */
-typedef	float gran_t;
+typedef	struct gran gran_t;
+
+
+/** \brief Calibration table struct */
+struct cal_table {
+	int size;	/*!< number of plots in the table */
+	struct {
+		int raw;	/*!< raw (A/D) value, as returned by \a RIG_LEVEL_RAWSTR */
+		int val;	/*!< associated value, basically the measured dB value */
+	} table[MAX_CAL_LENGTH];	/*!< table of plots */
+};
+
+/**
+ * \brief calibration table type
+ *
+ * cal_table_t is a data type suited to hold linear calibration.
+ * cal_table_t.size tell the number of plot cal_table_t.table contains.
+ *
+ * If a value is below or equal to cal_table_t.table[0].raw, 
+ * rig_raw2val() will return cal_table_t.table[0].val.
+ *
+ * If a value is greater or equal to cal_table_t.table[cal_table_t.size-1].raw, 
+ * rig_raw2val() will return cal_table_t.table[cal_table_t.size-1].val.
+ */
+typedef struct cal_table cal_table_t;
+
+#define EMPTY_STR_CAL { 0, { { 0, 0 }, } }
 
 
 /**
@@ -1015,6 +1072,8 @@ struct rig_caps {
   struct tuning_step_list tuning_steps[TSLSTSIZ];	/*!< Tuning step list */
   struct filter_list filters[FLTLSTSIZ];		/*!< mode/filter table, at -6dB */
 
+  cal_table_t str_cal;				/*!< S-meter calibration table */
+
   const struct confparams *cfgparams;            /*!< Configuration parametres. */
   const rig_ptr_t priv;                          /*!< Private data. */
 
@@ -1155,7 +1214,6 @@ typedef struct {
 	dcd_type_t dcd;		/*!< DCD port type */
   } type;
   int fd;			/*!< File descriptor */
-  FILE *stream;			/*!< FILE stream handle */
 #if defined(__CYGWIN__) || defined(_WIN32)
   HANDLE handle;		/*!< Win32 handle for serial special handling (PTT,DCD,..) */
 #endif
@@ -1181,10 +1239,7 @@ typedef struct {
 		int pin;	/*!< Parrallel port pin number */
 	} parallel;		/*!< parallel attributes */
 	struct {
-#if defined(__CYGWIN__) || defined(_WIN32)
-#else
-		struct sockaddr saddr;	/*!< Dest socket address */
-#endif
+		/* place holder */
 	} network;		/*!< Network attributes */
   } parm;			/*!< Port parameter union */
 } port_t;
@@ -1217,6 +1272,8 @@ struct rig_state {
 
   struct filter_list filters[FLTLSTSIZ];	/*!< Mode/filter table, at -6dB */
 
+  cal_table_t str_cal;				/*!< S-meter calibration table */
+
   chan_t chan_list[CHANLSTSIZ];	/*!< Channel list, zero ended */
 
   shortfreq_t max_rit;		/*!< max absolute RIT */
@@ -1245,12 +1302,17 @@ struct rig_state {
 
   int hold_decode;	/*!< set to 1 to hold the event decoder (async) otherwise 0 */
   vfo_t current_vfo;	/*!< VFO currently set */
-  int transceive;	/*!< Whether the transceive mode is on */
   int vfo_list;		/*!< Complete list of VFO for this rig */
   int comm_state;	/*!< Comm port state, opened/closed. */
   rig_ptr_t priv;	/*!< Pointer to private rig state data. */
   rig_ptr_t obj;	/*!< Internal use by hamlib++ for event handling. */
 
+  int transceive;	/*!< Whether the transceive mode is on */
+  int poll_interval;	/*!< Event notification polling period in milliseconds */
+  freq_t current_freq;	/*!< Frequency currently set */
+  rmode_t current_mode;	/*!< Mode currently set */
+  pbwidth_t current_width;	/*!< Passband width currently set */
+
 };
 
 
@@ -1259,6 +1321,7 @@ typedef int (*mode_cb_t) (RIG *, vfo_t, rmode_t, pbwidth_t, rig_ptr_t);
 typedef int (*vfo_cb_t) (RIG *, vfo_t, rig_ptr_t);
 typedef int (*ptt_cb_t) (RIG *, vfo_t, ptt_t, rig_ptr_t);
 typedef int (*dcd_cb_t) (RIG *, vfo_t, dcd_t, rig_ptr_t);
+typedef int (*pltune_cb_t) (RIG *, vfo_t, freq_t *, rmode_t *, pbwidth_t *, rig_ptr_t);
 
 /**
  * \brief Callback functions and args for rig event.
@@ -1288,6 +1351,8 @@ struct rig_callbacks {
   rig_ptr_t ptt_arg;	/*!< PTT change argument */
   dcd_cb_t dcd_event;	/*!< DCD change event */
   rig_ptr_t dcd_arg;	/*!< DCD change argument */
+  pltune_cb_t pltune;   /*!< Pipeline tuning module freq/mode/width callback */
+  rig_ptr_t pltune_arg; /*!< Pipeline tuning argument */
   /* etc.. */
 };
 
@@ -1443,6 +1508,7 @@ extern HAMLIB_EXPORT(int) rig_set_mode_callback HAMLIB_PARAMS((RIG *, mode_cb_t,
 extern HAMLIB_EXPORT(int) rig_set_vfo_callback HAMLIB_PARAMS((RIG *, vfo_cb_t, rig_ptr_t));
 extern HAMLIB_EXPORT(int) rig_set_ptt_callback HAMLIB_PARAMS((RIG *, ptt_cb_t, rig_ptr_t));
 extern HAMLIB_EXPORT(int) rig_set_dcd_callback HAMLIB_PARAMS((RIG *, dcd_cb_t, rig_ptr_t));
+extern HAMLIB_EXPORT(int) rig_set_pltune_callback HAMLIB_PARAMS((RIG *, pltune_cb_t, rig_ptr_t));
 
 extern HAMLIB_EXPORT(const char *) rig_get_info HAMLIB_PARAMS((RIG *rig));
 
