@@ -1,8 +1,8 @@
 /*
  *  Hamlib Interface - list of known rigs
- *  Copyright (c) 2000,2001 by Stephane Fillod and Frank Singleton
+ *  Copyright (c) 2000-2004 by Stephane Fillod and Frank Singleton
  *
- *		$Id: riglist.h,v 1.18 2001/08/12 22:59:59 f4cfe Exp $
+ *	$Id: riglist.h,v 1.43 2004/02/08 17:03:25 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -26,8 +26,31 @@
 #define RIG_MAKE_MODEL(a,b) ((a)*100+(b))
 #define RIG_BACKEND_NUM(a) ((a)/100)
 
+/*! \file riglist.h
+ *  \brief Hamlib rig(radio) model definitions.
+ *
+ *  This file contains rig model definitions for the Hamlib rig API.
+ *  Each distinct rig type has a unique model number (ID) and is used
+ *  by hamlib to identify and distinguish between the different hardware drivers.
+ *  The exact model numbers can be acquired using the macros in this
+ *  file. To obtain a list of supported rig branches, one can use the statically
+ *  defined RIG_BACKEND_LIST macro. To obtain a full list of supported rig (including
+ *  each model in every branch), the foreach_opened_rig() API function can be used.
+ *
+ *  The model number, or ID, is used to tell hamlib, which rig the client whishes to
+ *  use. It is done with the rig_init() API call.
+ */
+
 #define RIG_MODEL_NONE 0
 
+/*! \def RIG_MODEL_DUMMY
+ *  \brief A macro that returns the model number for the dummy backend.
+ *
+ *  The dummy backend, as the name suggests, is a backend which performs
+ *  no hardware operations and always behaves as one would expect. It can
+ *  be thought of as a hardware simulator and is very usefull for testing
+ *  client applications.
+ */
 #define RIG_DUMMY 0
 #define RIG_BACKEND_DUMMY "dummy"
 #define RIG_MODEL_DUMMY RIG_MAKE_MODEL(RIG_DUMMY, 1)
@@ -56,6 +79,10 @@
 #define RIG_MODEL_FRG100 RIG_MAKE_MODEL(RIG_YAESU, 17)
 #define RIG_MODEL_FRG9600 RIG_MAKE_MODEL(RIG_YAESU, 18)
 #define RIG_MODEL_FRG8800 RIG_MAKE_MODEL(RIG_YAESU, 19)
+#define RIG_MODEL_FT817 RIG_MAKE_MODEL(RIG_YAESU, 20)
+#define RIG_MODEL_FT100 RIG_MAKE_MODEL(RIG_YAESU, 21)
+#define RIG_MODEL_FT857 RIG_MAKE_MODEL(RIG_YAESU, 22)
+#define RIG_MODEL_FT897 RIG_MAKE_MODEL(RIG_YAESU, 23)
 
 	/*
 	 * Kenwood
@@ -78,14 +105,23 @@
 #define RIG_MODEL_TS2000 RIG_MAKE_MODEL(RIG_KENWOOD, 14)
 #define RIG_MODEL_R5000 RIG_MAKE_MODEL(RIG_KENWOOD, 15)
 #define RIG_MODEL_TS570S RIG_MAKE_MODEL(RIG_KENWOOD, 16)
+#define RIG_MODEL_THD7A RIG_MAKE_MODEL(RIG_KENWOOD, 17)
+#define RIG_MODEL_THD7AG RIG_MAKE_MODEL(RIG_KENWOOD, 18)
+#define RIG_MODEL_THF6A RIG_MAKE_MODEL(RIG_KENWOOD, 19)
+#define RIG_MODEL_THF7E RIG_MAKE_MODEL(RIG_KENWOOD, 20)
+#define RIG_MODEL_K2 RIG_MAKE_MODEL(RIG_KENWOOD, 21)
+#define RIG_MODEL_TS930 RIG_MAKE_MODEL(RIG_KENWOOD, 22)
+#define RIG_MODEL_THG71 RIG_MAKE_MODEL(RIG_KENWOOD, 23)
+#define RIG_MODEL_TS680S RIG_MAKE_MODEL(RIG_KENWOOD, 24)
+#define RIG_MODEL_TS140S RIG_MAKE_MODEL(RIG_KENWOOD, 25)
+#define RIG_MODEL_TMD700 RIG_MAKE_MODEL(RIG_KENWOOD, 26)
 
 	/*
 	 * Icom
 	 */
 #define RIG_ICOM 3
 #define RIG_BACKEND_ICOM "icom"
-	/* do-it-all, for debug purpose */
-#define RIG_MODEL_ICALL RIG_MAKE_MODEL(RIG_ICOM, 0)
+#define RIG_MODEL_ICALL RIG_MAKE_MODEL(RIG_ICOM, 0)	/* deprecated */
 #define RIG_MODEL_IC1271 RIG_MAKE_MODEL(RIG_ICOM, 1)
 #define RIG_MODEL_IC1275 RIG_MAKE_MODEL(RIG_ICOM, 2)
 #define RIG_MODEL_IC271 RIG_MAKE_MODEL(RIG_ICOM, 3)
@@ -129,17 +165,26 @@
 #define RIG_MODEL_ICR7100 RIG_MAKE_MODEL(RIG_ICOM, 41)
 #define RIG_MODEL_ICR8500 RIG_MAKE_MODEL(RIG_ICOM, 42)
 #define RIG_MODEL_ICR9000 RIG_MAKE_MODEL(RIG_ICOM, 43)
+#define RIG_MODEL_IC910 RIG_MAKE_MODEL(RIG_ICOM, 44)
+#define RIG_MODEL_IC78 RIG_MAKE_MODEL(RIG_ICOM, 45)
+#define RIG_MODEL_IC746PRO RIG_MAKE_MODEL(RIG_ICOM, 46)
+#define RIG_MODEL_IC756PROII RIG_MAKE_MODEL(RIG_ICOM, 47)
+#define RIG_MODEL_ICID1 RIG_MAKE_MODEL(RIG_ICOM, 54)
+#define RIG_MODEL_IC703 RIG_MAKE_MODEL(RIG_ICOM, 55)
+
 	/*
 	 * Optoelectronics (CI-V)
 	 */
-#define RIG_MODEL_MINISCOUT RIG_MAKE_MODEL(RIG_ICOM, 44)
-#define RIG_MODEL_XPLORER RIG_MAKE_MODEL(RIG_ICOM, 45)
+#define RIG_MODEL_MINISCOUT RIG_MAKE_MODEL(RIG_ICOM, 48)
+#define RIG_MODEL_XPLORER RIG_MAKE_MODEL(RIG_ICOM, 49)
+#define RIG_MODEL_OS535 RIG_MAKE_MODEL(RIG_ICOM, 52)
+#define RIG_MODEL_OS456 RIG_MAKE_MODEL(RIG_ICOM, 53)
 
 	/*
 	 * TenTec (CI-V)
 	 */
-#define RIG_MODEL_OMNIVI RIG_MAKE_MODEL(RIG_ICOM, 46)
-#define RIG_MODEL_OMNIVIP RIG_MAKE_MODEL(RIG_ICOM, 47) /* OMNI-IV+ */
+#define RIG_MODEL_OMNIVI RIG_MAKE_MODEL(RIG_ICOM, 50)
+#define RIG_MODEL_OMNIVIP RIG_MAKE_MODEL(RIG_ICOM, 51) /* OMNI-VI+ */
 
 	/*
 	 * Icom PCR
@@ -209,6 +254,7 @@
 #define RIG_BACKEND_DRAKE "drake"
 #define RIG_MODEL_DKR8 RIG_MAKE_MODEL(RIG_DRAKE, 1)
 #define RIG_MODEL_DKR8A RIG_MAKE_MODEL(RIG_DRAKE, 2)
+#define RIG_MODEL_DKR8B RIG_MAKE_MODEL(RIG_DRAKE, 3)
 
 	/*
 	 * Lowe
@@ -218,6 +264,7 @@
 #define RIG_MODEL_HF150 RIG_MAKE_MODEL(RIG_LOWE, 1)
 #define RIG_MODEL_HF225 RIG_MAKE_MODEL(RIG_LOWE, 2)
 #define RIG_MODEL_HF250 RIG_MAKE_MODEL(RIG_LOWE, 3)
+#define RIG_MODEL_HF235 RIG_MAKE_MODEL(RIG_LOWE, 4)
 
 	/*
 	 * Racal
@@ -279,6 +326,10 @@
 #define RIG_MODEL_TT538 RIG_MAKE_MODEL(RIG_TENTEC, 2)	/* Jupiter */
 #define RIG_MODEL_RX320 RIG_MAKE_MODEL(RIG_TENTEC, 3)
 #define RIG_MODEL_RX340 RIG_MAKE_MODEL(RIG_TENTEC, 4)
+#define RIG_MODEL_RX350 RIG_MAKE_MODEL(RIG_TENTEC, 5)
+#define RIG_MODEL_TT526 RIG_MAKE_MODEL(RIG_TENTEC, 6)	/* 6N2 */
+#define RIG_MODEL_TT516 RIG_MAKE_MODEL(RIG_TENTEC, 7)	/* Argonaut V */
+#define RIG_MODEL_TT565 RIG_MAKE_MODEL(RIG_TENTEC, 8)	/* Orion */
 
 	/*
 	 * Alinco
@@ -294,16 +345,80 @@
 #define RIG_BACKEND_KACHINA "kachina"
 #define RIG_MODEL_505DSP RIG_MAKE_MODEL(RIG_KACHINA, 1)
 
+
+/*! \def RIG_MODEL_RPC
+ *  \brief A macro that returns the model number of the RPC Network pseudo-backend.
+ *
+ *  The RPC backend can be used to connect and send commands to a rig server,
+ *  \c rpc.rigd, running on a remote machine. Using this client/server scheme,
+ *  several clients can control and monitor the same rig hardware.
+ */
+#define RIG_RPC 19
+#define RIG_BACKEND_RPC "rpcrig"
+#define RIG_MODEL_RPC RIG_MAKE_MODEL(RIG_RPC, 1)
+
+	/*
+	 * Gnuradio backend
+	 */
+#define RIG_GNURADIO 20
+#define RIG_BACKEND_GNURADIO "gnuradio"
+#define RIG_MODEL_GNURADIO RIG_MAKE_MODEL(RIG_GNURADIO, 1) /* dev model, Chirp source */
+#define RIG_MODEL_MC4020 RIG_MAKE_MODEL(RIG_GNURADIO, 2)	/* MC4020 */
+#define RIG_MODEL_GRAUDIO RIG_MAKE_MODEL(RIG_GNURADIO, 3)	/* Sound card source */
+#define RIG_MODEL_GRAUDIOIQ RIG_MAKE_MODEL(RIG_GNURADIO, 4)	/* I&Q stereo sound card source */
+#define RIG_MODEL_USRP RIG_MAKE_MODEL(RIG_GNURADIO, 5)	/* Universal Software Radio Peripheral */
+
+	/*
+	 * Microtune tuners
+	 */
+#define RIG_MICROTUNE 21
+#define RIG_BACKEND_MICROTUNE "microtune"
+#define RIG_MODEL_MICROTUNE_4937 RIG_MAKE_MODEL(RIG_MICROTUNE, 1)	/* eval board */
+#define RIG_MODEL_MICROTUNE_4702 RIG_MAKE_MODEL(RIG_MICROTUNE, 2)	/* Alan's */
+
+	/*
+	 * TAPR
+	 */
+#define RIG_TAPR 22
+#define RIG_BACKEND_TAPR "tapr"
+#define RIG_MODEL_DSP10 RIG_MAKE_MODEL(RIG_TAPR, 1)
+
+	/*
+	 * Flex-radio
+	 */
+#define RIG_FLEXRADIO 23
+#define RIG_BACKEND_FLEXRADIO "flexradio"
+#define RIG_MODEL_SDR1000 RIG_MAKE_MODEL(RIG_FLEXRADIO, 1)
+
+
+	/*
+	 * VEB Funkwerk Kpenick RFT
+	 */
+#define RIG_RFT 24
+#define RIG_BACKEND_RFT "rft"
+#define RIG_MODEL_EKD500 RIG_MAKE_MODEL(RIG_RFT, 1)
+
+
 	/*
 	 * TODO:
 		RIG_MODEL_KWZ30,	KNEISNER +DOERING
 		RIG_MODEL_E1800,	DASA-Telefunken
-		RIG_EKD500,	RFT
-		RIG_W41PC (ISA card) Wavecom
+		etc.
 	*/
 
+/*! \typedef typedef int rig_model_t
+    \brief Convenience type definition for rig model.
+*/
 typedef int rig_model_t;
 
+
+/*! \def RIG_BACKEND_LIST
+ *  \brief Static list of rig models.
+ *
+ *  This is a NULL terminated list of available rig backends. Each entry
+ *  in the list consists of two fields: The branch number, which is an integer,
+ *  and the branch name, which is a character string.
+ */
 #define RIG_BACKEND_LIST {		\
 		{ RIG_DUMMY, RIG_BACKEND_DUMMY }, \
 		{ RIG_YAESU, RIG_BACKEND_YAESU }, \
@@ -324,6 +439,12 @@ typedef int rig_model_t;
 		{ RIG_TENTEC, RIG_BACKEND_TENTEC }, \
 		{ RIG_ALINCO, RIG_BACKEND_ALINCO }, \
 		{ RIG_KACHINA, RIG_BACKEND_KACHINA }, \
+		{ RIG_RPC, RIG_BACKEND_RPC }, \
+		{ RIG_GNURADIO, RIG_BACKEND_GNURADIO }, \
+		{ RIG_MICROTUNE, RIG_BACKEND_MICROTUNE }, \
+		{ RIG_TAPR, RIG_BACKEND_TAPR }, \
+		{ RIG_FLEXRADIO, RIG_BACKEND_FLEXRADIO }, \
+		{ RIG_RFT, RIG_BACKEND_RFT }, \
 		{ 0, NULL }, /* end */  \
 }
 
