@@ -2,7 +2,7 @@
  *  Hamlib JRC backend - main file
  *  Copyright (c) 2001-2003 by Stephane Fillod
  *
- *	$Id: jrc.c,v 1.8 2003/04/16 22:30:40 fillods Exp $
+ *	$Id: jrc.c,v 1.11 2003/11/16 17:14:43 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
@@ -122,7 +123,7 @@ int jrc_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 		if (freq >= GHz(10))
 				return -RIG_EINVAL;
 
-		freq_len = sprintf(freqbuf, "F%10Ld" EOM, freq);
+		freq_len = sprintf(freqbuf, "F%10Ld" EOM, (long long)freq);
 
 		return jrc_transaction (rig, freqbuf, freq_len, NULL, NULL);
 }
@@ -135,6 +136,7 @@ int jrc_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 {
 		int freq_len, retval;
 		char freqbuf[BUFSZ];
+		long long f;
 
 		retval = jrc_transaction (rig, "F" EOM, 2, freqbuf, &freq_len);
 		if (retval != RIG_OK)
@@ -148,7 +150,8 @@ int jrc_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 		freqbuf[freq_len-1] = '\0';
 
 		/* extract freq */
-		sscanf(freqbuf+1, "%llu", freq);
+		sscanf(freqbuf+1, "%llu", &f);
+		*freq = f;
 
 		return RIG_OK;
 }
@@ -410,17 +413,14 @@ int jrc_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
  */
 int jrc_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 {
- 		struct jrc_priv_caps *priv;
 		int retval, lvl_len, lvl;
 		char lvlbuf[BUFSZ];
 
-		priv = (struct jrc_priv_caps*)rig->caps->priv;
-
 		/* Optimize:
 		 *   sort the switch cases with the most frequent first
 		 */
 		switch (level) {
-		case RIG_LEVEL_STRENGTH:
+		case RIG_LEVEL_RAWSTR:
 				/* read A/D converted value */
 			retval = jrc_transaction (rig, "M" EOM, 2, lvlbuf, &lvl_len);
 			if (retval != RIG_OK)
@@ -433,7 +433,7 @@ int jrc_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 			}
 
 			lvlbuf[4] = '\0';
-			val->i = rig_raw2val(atoi(lvlbuf+1), &priv->str_cal);
+			val->i = atoi(lvlbuf+1);
 			break;
 
 		case RIG_LEVEL_SQLSTAT:
@@ -576,12 +576,9 @@ int jrc_set_parm(RIG *rig, setting_t parm, value_t val)
  */
 int jrc_get_parm(RIG *rig, setting_t parm, value_t *val)
 {
- 		struct jrc_priv_caps *priv;
 		int retval, lvl_len, i;
 		char lvlbuf[BUFSZ];
 
-		priv = (struct jrc_priv_caps*)rig->caps->priv;
-
 		/* Optimize:
 		 *   sort the switch cases with the most frequent first
 		 */
@@ -838,8 +835,11 @@ int jrc_decode_event(RIG *rig)
 		 */
 
 		if (rig->callbacks.freq_event) {
+			long long f;
+
 			buf[14] = '\0';	/* side-effect: destroy AGC first digit! */
-			sscanf(buf+4, "%lld", &freq);
+			sscanf(buf+4, "%lld", &f);
+			freq = f;
 			return rig->callbacks.freq_event(rig, RIG_VFO_CURR, freq,
 							rig->callbacks.freq_arg);
 		}
@@ -861,6 +861,7 @@ DECLARE_INITRIG_BACKEND(jrc)
 {
 		rig_debug(RIG_DEBUG_VERBOSE, "jrc: _init called\n");
 
+		rig_register(&nrd535_caps);
 		rig_register(&nrd545_caps);
 
 		return RIG_OK;
