@@ -1,8 +1,8 @@
 /*
  *  Hamlib Kenwood backend - main file
- *  Copyright (c) 2000-2003 by Stephane Fillod and others
+ *  Copyright (c) 2000-2004 by Stephane Fillod and others
  *
- *	$Id: kenwood.c,v 1.61 2003/04/16 22:30:41 fillods Exp $
+ *	$Id: kenwood.c,v 1.73 2004/02/14 16:50:53 f4dwv Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -25,6 +25,7 @@
 #endif
 
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
 #include <math.h>
@@ -76,6 +77,8 @@ static const struct kenwood_id kenwood_id_list[] = {
 	{ RIG_MODEL_TS440, 4 },
 	{ RIG_MODEL_R5000, 5 },
 	{ RIG_MODEL_TS790, 7 },
+        { RIG_MODEL_TS950SDX, 8 },	/* reported as RIG_MODEL_TS950SD originally */
+        { RIG_MODEL_TS850, 9 },
 	{ RIG_MODEL_TS450S, 10 },
 	{ RIG_MODEL_TS690S, 11 },
 	{ RIG_MODEL_TS870S, 15 },
@@ -88,8 +91,10 @@ static const struct kenwood_id kenwood_id_list[] = {
 static const struct kenwood_id_string kenwood_id_string_list[] = {
 	{ RIG_MODEL_THD7A,  "TH-D7" },
 	{ RIG_MODEL_THD7AG, "TH-D7G" },
+	{ RIG_MODEL_TMD700, "TM-D700" },
 	{ RIG_MODEL_THF6A, "TH-F6" },
 	{ RIG_MODEL_THF7E, "TH-F7" },
+	{ RIG_MODEL_THG71, "TH-G71" },
 	{ RIG_MODEL_NONE, NULL },	/* end marker */
 };
 
@@ -160,11 +165,12 @@ kenwood_transaction (RIG *rig, const char *cmdstr, int cmd_len,
     }
 
 transaction_read:
+    memset(data,0,*datasize);
     retval = read_string(&rs->rigport, data, *datasize, cmdtrm, strlen(cmdtrm));
     if (retval < 0)
-        goto transaction_quit;
+        	goto transaction_quit;
 	else
-			*datasize = retval;
+		*datasize = retval;
 
     /* Check that command termination is correct */
     if (!strchr(cmdtrm, data[strlen(data)])) {
@@ -262,42 +268,87 @@ int kenwood_set_vfo(RIG *rig, vfo_t vfo)
 		return retval;
 }
 
+int kenwood_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t txvfo)
+{
+		unsigned char cmdbuf[16], ackbuf[16];
+		int cmd_len, ack_len, retval;
+		char vfo_function;
+
+		if(vfo !=RIG_VFO_CURR) {
+		switch (vfo) {
+		case RIG_VFO_VFO:
+		case RIG_VFO_A: vfo_function = '0'; break;
+		case RIG_VFO_B: vfo_function = '1'; break;
+		case RIG_VFO_MEM: vfo_function = '2'; break;
+		default: 
+			rig_debug(RIG_DEBUG_ERR,"kenwood_set_split_vfo: unsupported VFO %d\n",
+								vfo);
+			return -RIG_EINVAL;
+		}
+		
+		/* set RX VFO */
+		cmd_len = sprintf(cmdbuf, "FR%c%s", vfo_function, cmd_trm(rig));
+		ack_len = 0;
+		retval = kenwood_transaction (rig, cmdbuf, cmd_len, ackbuf, &ack_len);
+		if (retval != RIG_OK)
+			return retval;
+		}
+
+		if(split==RIG_SPLIT_ON) {
+			switch (txvfo) {
+				case RIG_VFO_VFO:
+				case RIG_VFO_A: vfo_function = '0'; break;
+				case RIG_VFO_B: vfo_function = '1'; break;
+				case RIG_VFO_MEM: vfo_function = '2'; break;
+				default: 
+					rig_debug(RIG_DEBUG_ERR,"kenwood_set_split_vfo: unsupported VFO %d\n", txvfo);
+					return -RIG_EINVAL;
+			} 
+		} else
+		  if(vfo==RIG_VFO_CURR)
+			rig_debug(RIG_DEBUG_ERR,"kenwood_set_split_vfo: unsupported VFO %d\n", vfo);
+
+		/* set TX VFO */
+		cmd_len = sprintf(cmdbuf, "FT%c%s", vfo_function, cmd_trm(rig));
+		ack_len = 0;
+		retval = kenwood_transaction (rig, cmdbuf, cmd_len, ackbuf, &ack_len);
+		return retval;
+}
 
 /*
- * kenwood_get_vfo
+ * kenwood_get_vfo using byte 31 of the IF information field
  * Assumes rig!=NULL, !vfo
  */
 int kenwood_get_vfo(RIG *rig, vfo_t *vfo)
 {
-		unsigned char vfobuf[50];
-		int vfo_len, retval;
+		unsigned char infobuf[50];
+		int info_len, retval;
 
-
-		/* query RX VFO */
-		vfo_len = 50;
-		retval = kenwood_transaction (rig, "FR;", 3, vfobuf, &vfo_len);
+                info_len = 50;
+		retval = kenwood_transaction (rig, "IF;", 3, infobuf, &info_len);
 		if (retval != RIG_OK)
 			return retval;
 
-		if (vfo_len != 4 || vfobuf[1] != 'R') {
-			rig_debug(RIG_DEBUG_ERR,"kenwood_get_vfo: unexpected answer %s, "
-							"len=%d\n", vfobuf, vfo_len);
-			return -RIG_ERJCTED;
-		}
+		if (info_len != 38 || infobuf[1] != 'F') {
+		rig_debug(RIG_DEBUG_ERR,"kenwood_get_vfo: wrong answer len=%d\n",
+						info_len);
+		return -RIG_ERJCTED;
 
-		/* TODO: replace 0,1,2,.. constants by defines */
-		switch (vfobuf[2]) {
+		}
+		switch (infobuf[30]) {
 		case '0': *vfo = RIG_VFO_A; break;
 		case '1': *vfo = RIG_VFO_B; break;
 		case '2': *vfo = RIG_VFO_MEM; break;
 		default: 
 			rig_debug(RIG_DEBUG_ERR,"kenwood_get_vfo: unsupported VFO %c\n",
-								vfobuf[2]);
+								infobuf[30]);
 			return -RIG_EPROTO;
 		}
 		return RIG_OK;
+
 }
 
+
 /*
  * kenwood_old_set_vfo
  * Assumes rig!=NULL
@@ -397,7 +448,7 @@ int kenwood_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 								vfo);
 			return -RIG_EINVAL;
 		}
-		freq_len = sprintf(freqbuf,"F%c%011Ld;", vfo_letter, freq);
+		freq_len = sprintf(freqbuf,"F%c%011Ld;", vfo_letter, (long long)freq);
 
 		ack_len = 0;
 		retval = kenwood_transaction (rig, freqbuf, freq_len, ackbuf, &ack_len);
@@ -415,6 +466,7 @@ int kenwood_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 		unsigned char cmdbuf[4];
 		int cmd_len, freq_len, retval;
 		char vfo_letter;
+		long long f;
 
 			/*
 			 * better FIXME: vfo==RIG_VFO_CURR
@@ -448,7 +500,8 @@ int kenwood_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 			return -RIG_ERJCTED;
 		}
 
-		sscanf(freqbuf+2, "%lld", freq);
+		sscanf(freqbuf+2, "%lld", &f);
+		*freq = (freq_t)f;
 
 		return RIG_OK;
 }
@@ -1487,19 +1540,27 @@ DECLARE_INITRIG_BACKEND(kenwood)
 
 		rig_register(&ts950sdx_caps);
 		rig_register(&ts50s_caps);
+		rig_register(&ts140_caps);
 		rig_register(&ts440_caps);
 		rig_register(&ts450s_caps);
 		rig_register(&ts570d_caps);
 		rig_register(&ts570s_caps);
+		rig_register(&ts680s_caps);
+		rig_register(&ts690s_caps);
+		rig_register(&ts711_caps);
+		rig_register(&ts811_caps);
 		rig_register(&ts790_caps);
 		rig_register(&ts850_caps);
 		rig_register(&ts870s_caps);
 		rig_register(&ts930_caps);
+		rig_register(&ts940_caps);
 		rig_register(&ts2000_caps);
 		rig_register(&k2_caps);
 
+		rig_register(&tmd700_caps);
 		rig_register(&thd7a_caps);
 		rig_register(&thf7e_caps);
+		rig_register(&thg71_caps);
 
 		return RIG_OK;
 }
