@@ -1,8 +1,8 @@
 /*
  *  Hamlib Kenwood backend - main header
- *  Copyright (c) 2000-2003 by Stephane Fillod
+ *  Copyright (c) 2000-2004 by Stephane Fillod
  *
- *	$Id: kenwood.h,v 1.29 2003/04/16 22:30:41 fillods Exp $
+ *	$Id: kenwood.h,v 1.34 2004/02/14 16:50:54 f4dwv Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -49,6 +49,7 @@ int kenwood_transaction(RIG *rig, const char *cmd, int cmd_len, char *data,
 				size_t *data_len);
 int kenwood_set_vfo(RIG *rig, vfo_t vfo);
 int kenwood_get_vfo(RIG *rig, vfo_t *vfo);
+int kenwood_set_split_vfo(RIG *rig, vfo_t vfo , split_t split, vfo_t txvfo);
 int kenwood_old_set_vfo(RIG *rig, vfo_t vfo);
 int kenwood_old_get_vfo(RIG *rig, vfo_t *vfo);
 int kenwood_set_freq(RIG *rig, vfo_t vfo, freq_t freq);
@@ -82,18 +83,26 @@ int kenwood_get_trn(RIG *rig, int *trn);
 
 extern const struct rig_caps ts950sdx_caps;
 extern const struct rig_caps ts50s_caps;
+extern const struct rig_caps ts140_caps;
 extern const struct rig_caps ts440_caps;
 extern const struct rig_caps ts450s_caps;
 extern const struct rig_caps ts570d_caps;
 extern const struct rig_caps ts570s_caps;
+extern const struct rig_caps ts680s_caps;
+extern const struct rig_caps ts690s_caps;
+extern const struct rig_caps ts711_caps;
+extern const struct rig_caps ts811_caps;
 extern const struct rig_caps ts790_caps;
 extern const struct rig_caps ts850_caps;
 extern const struct rig_caps ts870s_caps;
 extern const struct rig_caps ts930_caps;
+extern const struct rig_caps ts940_caps;
 extern const struct rig_caps ts2000_caps;
 extern const struct rig_caps thd7a_caps;
+extern const struct rig_caps tmd700_caps;
 extern const struct rig_caps thf7a_caps;
 extern const struct rig_caps thf7e_caps;
+extern const struct rig_caps thg71_caps;
 extern const struct rig_caps k2_caps;
 
 
