@@ -2,7 +2,7 @@
  *  Hamlib Interface - toolbox
  *  Copyright (c) 2000-2003 by Stephane Fillod and Frank Singleton
  *
- *	$Id: misc.c,v 1.25 2003/03/24 23:07:10 fillods Exp $
+ *	$Id: misc.c,v 1.31 2003/12/24 09:07:52 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -292,131 +292,13 @@ const char *strstatus(enum rig_status_e status)
 	return "";
 }
 
-int sprintf_mode(char *str, rmode_t mode)
-{
-		int i, len=0;
-
-		*str = '\0';
-		if (mode == RIG_MODE_NONE)
-				return 0;
-
-		for (i = 0; i < 30; i++) {
-				const char *ms = strrmode(mode & (1UL<<i));
-				if (!ms || !ms[0])
-						continue;	/* unknown, FIXME! */
-				strcat(str, ms);
-				strcat(str, " ");
-				len += strlen(ms) + 1;
-		}
-		return len;
-}
-
-int sprintf_func(char *str, setting_t func)
-{
-		int i, len=0;
-
-		*str = '\0';
-		if (func == RIG_FUNC_NONE)
-				return 0;
-
-		for (i = 0; i < 60; i++) {
-				const char *ms = strfunc(func & rig_idx2setting(i));
-				if (!ms || !ms[0])
-						continue;	/* unknown, FIXME! */
-				strcat(str, ms);
-				strcat(str, " ");
-				len += strlen(ms) + 1;
-		}
-		return len;
-}
-
-
-int sprintf_level(char *str, setting_t level)
-{
-		int i, len=0;
-
-		*str = '\0';
-		if (level == RIG_LEVEL_NONE)
-				return 0;
-
-		for (i = 0; i < 60; i++) {
-				const char *ms = strlevel(level & rig_idx2setting(i));
-				if (!ms || !ms[0])
-						continue;	/* unknown, FIXME! */
-				strcat(str, ms);
-				strcat(str, " ");
-				len += strlen(ms) + 1;
-		}
-		return len;
-}
-
-
-int sprintf_parm(char *str, setting_t parm)
-{
-		int i, len=0;
-
-		*str = '\0';
-		if (parm == RIG_PARM_NONE)
-				return 0;
-
-		for (i = 0; i < 60; i++) {
-				const char *ms = strparm(parm & rig_idx2setting(i));
-				if (!ms || !ms[0])
-						continue;	/* unknown, FIXME! */
-				strcat(str, ms);
-				strcat(str, " ");
-				len += strlen(ms) + 1;
-		}
-		return len;
-}
-
-
-int sprintf_vfop(char *str, vfo_op_t op)
-{
-		int i, len=0;
-
-		*str = '\0';
-		if (op == RIG_OP_NONE)
-				return 0;
-
-		for (i = 0; i < 30; i++) {
-				const char *ms = strvfop(op & (1UL<<i));
-				if (!ms || !ms[0])
-						continue;	/* unknown, FIXME! */
-				strcat(str, ms);
-				strcat(str, " ");
-				len += strlen(ms) + 1;
-		}
-		return len;
-}
-
-
-int sprintf_scan(char *str, scan_t rscan)
-{
-		int i, len=0;
-
-		*str = '\0';
-		if (rscan == RIG_SCAN_NONE)
-				return 0;
-
-		for (i = 0; i < 30; i++) {
-				const char *ms = strscan(rscan & (1UL<<i));
-				if (!ms || !ms[0])
-						continue;	/* unknown, FIXME! */
-				strcat(str, ms);
-				strcat(str, " ");
-				len += strlen(ms) + 1;
-		}
-		return len;
-}
-
-
 
 static struct { 
 		rmode_t mode;
 		const char *str;
 } mode_str[] = {
 	{ RIG_MODE_AM, "AM" },
+	{ RIG_MODE_AMS, "AMS" },
 	{ RIG_MODE_FM, "FM" },
 	{ RIG_MODE_CW, "CW" },
 	{ RIG_MODE_CWR, "CWR" },
@@ -425,7 +307,10 @@ static struct {
 	{ RIG_MODE_RTTY, "RTTY" },
 	{ RIG_MODE_RTTYR, "RTTYR" },
 	{ RIG_MODE_WFM, "WFM" },
-	{ RIG_MODE_NONE, NULL },
+	{ RIG_MODE_PKTLSB, "PKTLSB" },
+	{ RIG_MODE_PKTUSB, "PKTUSB" },
+	{ RIG_MODE_PKTFM, "PKTFM" },
+	{ RIG_MODE_NONE, "" },
 };
 
 
@@ -433,7 +318,7 @@ rmode_t parse_mode(const char *s)
 {
 	int i;
 
-	for (i=0 ; mode_str[i].str != NULL; i++)
+	for (i=0 ; mode_str[i].str != ""; i++)
 			if (!strcmp(s, mode_str[i].str))
 					return mode_str[i].mode;
 	return RIG_MODE_NONE;
@@ -446,16 +331,16 @@ const char * strrmode(rmode_t mode)
 	if (mode == RIG_MODE_NONE)
 		return "";
 
-	for (i=0; mode_str[i].str != NULL; i++)
+	for (i=0; mode_str[i].str != ""; i++)
 		if (mode == mode_str[i].mode)
 			return mode_str[i].str;
 
-	return NULL;
+	return "";
 }
 
 
 static struct { 
-		vfo_t vfo ;
+		vfo_t vfo;
 		const char *str;
 } vfo_str[] = {
 		{ RIG_VFO_A, "VFOA" },
@@ -466,14 +351,14 @@ static struct {
 		{ RIG_VFO_VFO, "VFO" },
 		{ RIG_VFO_MAIN, "Main" },
 		{ RIG_VFO_SUB, "Sub" },
-		{ RIG_VFO_NONE, NULL },
+		{ RIG_VFO_NONE, "" },
 };
 
 vfo_t parse_vfo(const char *s)
 {
 	int i;
 
-	for (i=0; vfo_str[i].str != NULL; i++)
+	for (i=0; vfo_str[i].str != ""; i++)
 		if (!strcmp(s, vfo_str[i].str))
 			return vfo_str[i].vfo;
 	return RIG_VFO_NONE;
@@ -486,11 +371,11 @@ const char *strvfo(vfo_t vfo)
 	if (vfo == RIG_VFO_NONE)
 		return "";
 
-	for (i=0; vfo_str[i].str != NULL; i++)
+	for (i=0; vfo_str[i].str != ""; i++)
 		if (vfo == vfo_str[i].vfo)
 			return vfo_str[i].str;
 
-	return NULL;
+	return "";
 }
 
 
@@ -526,14 +411,14 @@ static struct {
 	{ RIG_FUNC_RESUME, "RESUME" },
 	{ RIG_FUNC_TBURST, "TBURST" },
 	{ RIG_FUNC_TUNER, "TUNER" },
-	{ RIG_FUNC_NONE, NULL },
+	{ RIG_FUNC_NONE, "" },
 };
 
 setting_t parse_func(const char *s)
 {
 	int i;
 
-	for (i=0 ; func_str[i].str != NULL; i++)
+	for (i=0 ; func_str[i].str != ""; i++)
 			if (!strcmp(s, func_str[i].str))
 					return func_str[i].func;
 	return RIG_FUNC_NONE;
@@ -546,11 +431,11 @@ const char *strfunc(setting_t func)
 	if (func == RIG_FUNC_NONE)
 		return "";
 
-	for (i=0; func_str[i].str != NULL; i++)
+	for (i=0; func_str[i].str != ""; i++)
 		if (func == func_str[i].func)
 			return func_str[i].str;
 
-	return NULL;
+	return "";
 }
 
 
@@ -581,19 +466,19 @@ static struct {
 	{ RIG_LEVEL_METER, "METER" },
 	{ RIG_LEVEL_VOXGAIN, "VOXGAIN" },
 	{ RIG_LEVEL_ANTIVOX, "ANTIVOX" },
-
+	{ RIG_LEVEL_RAWSTR, "RAWSTR" },
 	{ RIG_LEVEL_SWR, "SWR" },
 	{ RIG_LEVEL_ALC, "ALC" },
 	{ RIG_LEVEL_SQLSTAT, "SQLSTAT" },
 	{ RIG_LEVEL_STRENGTH, "STRENGTH" },
-	{ RIG_LEVEL_NONE, NULL },
+	{ RIG_LEVEL_NONE, "" },
 };
 
 setting_t parse_level(const char *s)
 {
 	int i;
 
-	for (i=0 ; level_str[i].str != NULL; i++)
+	for (i=0 ; level_str[i].str != ""; i++)
 			if (!strcmp(s, level_str[i].str))
 					return level_str[i].level;
 	return RIG_LEVEL_NONE;
@@ -606,11 +491,11 @@ const char *strlevel(setting_t level)
 	if (level == RIG_LEVEL_NONE)
 		return "";
 
-	for (i=0; level_str[i].str != NULL; i++)
+	for (i=0; level_str[i].str != ""; i++)
 		if (level == level_str[i].level)
 			return level_str[i].str;
 
-	return NULL;
+	return "";
 }
 
 
@@ -624,14 +509,14 @@ static struct {
 	{ RIG_PARM_BEEP, "BEEP" },
 	{ RIG_PARM_TIME, "TIME" },
 	{ RIG_PARM_BAT, "BAT" },
-	{ RIG_PARM_NONE, NULL },
+	{ RIG_PARM_NONE, "" },
 };
 
 setting_t parse_parm(const char *s)
 {
 	int i;
 
-	for (i=0 ; parm_str[i].str != NULL; i++)
+	for (i=0 ; parm_str[i].str != ""; i++)
 			if (!strcmp(s, parm_str[i].str))
 					return parm_str[i].parm;
 	return RIG_PARM_NONE;
@@ -644,11 +529,11 @@ const char *strparm(setting_t parm)
 	if (parm == RIG_PARM_NONE)
 		return "";
 
-	for (i=0; parm_str[i].str != NULL; i++)
+	for (i=0; parm_str[i].str != ""; i++)
 		if (parm == parm_str[i].parm)
 			return parm_str[i].str;
 
-	return NULL;
+	return "";
 }
 
 
@@ -668,14 +553,15 @@ static struct {
 	{ RIG_OP_LEFT, "LEFT" },
 	{ RIG_OP_RIGHT, "RIGHT" },
 	{ RIG_OP_TUNE, "TUNE" },
-	{ RIG_OP_NONE, NULL },
+	{ RIG_OP_TOGGLE, "TOGGLE" },
+	{ RIG_OP_NONE, "" },
 };
 
 vfo_op_t parse_vfo_op(const char *s)
 {
 	int i;
 
-	for (i=0 ; vfo_op_str[i].str != NULL; i++)
+	for (i=0 ; vfo_op_str[i].str != ""; i++)
 			if (!strcmp(s, vfo_op_str[i].str))
 					return vfo_op_str[i].vfo_op;
 	return RIG_OP_NONE;
@@ -688,11 +574,11 @@ const char *strvfop(vfo_op_t op)
 	if (op == RIG_OP_NONE)
 		return "";
 
-	for (i=0; vfo_op_str[i].str != NULL; i++)
+	for (i=0; vfo_op_str[i].str != ""; i++)
 		if (op == vfo_op_str[i].vfo_op)
 			return vfo_op_str[i].str;
 
-	return NULL;
+	return "";
 }
 
 
@@ -707,14 +593,15 @@ static struct {
 	{ RIG_SCAN_PROG, "PROG" },
 	{ RIG_SCAN_DELTA, "DELTA" },
 	{ RIG_SCAN_VFO, "VFO" },
-	{ RIG_SCAN_NONE, NULL },
+	{ RIG_SCAN_PLT, "PLT" },
+	{ RIG_SCAN_NONE, "" },
 };
 
 scan_t parse_scan(const char *s)
 {
 	int i;
 
-	for (i=0 ; scan_str[i].str != NULL; i++) {
+	for (i=0 ; scan_str[i].str != ""; i++) {
 		if (strcmp(s, scan_str[i].str) == 0) {
 			return scan_str[i].rscan;
 		}
@@ -730,11 +617,11 @@ const char *strscan(scan_t rscan)
 	if (rscan == RIG_SCAN_NONE)
 		return "";
 
-	for (i=0; scan_str[i].str != NULL; i++)
+	for (i=0; scan_str[i].str != ""; i++)
 		if (rscan == scan_str[i].rscan)
 			return scan_str[i].str;
 
-	return NULL;
+	return "";
 }
 
 
