@@ -4,7 +4,7 @@
  *  Parts of the PTT handling are derived from soundmodem, an excellent
  *  ham packet softmodem written by Thomas Sailer, HB9JNX.
  *
- *	$Id: serial.c,v 1.31 2003/04/19 11:49:43 fillods Exp $
+ *	$Id: serial.c,v 1.38 2003/10/01 19:44:00 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -58,6 +58,7 @@
 /* for CTS/RTS and DTR/DSR control under Win32 --SF */
 #ifdef HAVE_WINDOWS_H
 #include <windows.h>
+#include "par_nt.h"
 #endif
 #ifdef HAVE_WINIOCTL_H
 #include <winioctl.h>
@@ -66,6 +67,13 @@
 #include <winbase.h>
 #endif
 
+#if defined(WIN32) && !defined(HAVE_TERMIOS_H)
+#include "win32termios.h"
+#else
+#define OPEN open
+#define CLOSE close
+#endif
+
 #include <hamlib/rig.h>
 #include "serial.h"
 #include "misc.h"
@@ -76,6 +84,22 @@
 
 #ifdef HAVE_LINUX_PPDEV_H
 #include <linux/ppdev.h>
+#include <linux/parport.h>
+
+/* 
+ * These control port bits are active low.
+ * We toggle them so that this weirdness doesn't get get propagated
+ * through our interface.
+ */
+#define CP_ACTIVE_LOW_BITS	0x0B
+
+/*
+ * These status port bits are active low.
+ * We toggle them so that this weirdness doesn't get get propagated
+ * through our interface.
+ */
+#define SP_ACTIVE_LOW_BITS	0x80
+
 #endif
 
 
@@ -103,7 +127,7 @@ int serial_open(port_t *rp) {
   /*
    * Open in Non-blocking mode. Watch for EAGAIN errors!
    */
-  fd = open(rp->pathname, O_RDWR | O_NOCTTY | O_NDELAY);
+  fd = OPEN(rp->pathname, O_RDWR | O_NOCTTY | O_NDELAY);
 
   if (fd == -1) {
     
@@ -118,18 +142,10 @@ int serial_open(port_t *rp) {
 
   err = serial_setup(rp);
   if (err != RIG_OK) {
-		  close(fd);
+		  CLOSE(fd);
 		  return err;
   }
 
-  rp->stream = fdopen(fd, "r+b");
-  if (rp->stream == NULL) {
-		rig_debug(RIG_DEBUG_ERR, "open_serial: fdopen failed: %s\n", 
-					strerror(errno));
-		close(fd);
-		return -RIG_EIO;		/* arg, so close! */
-  }
-
   return RIG_OK;
 }
 
@@ -137,7 +153,8 @@ int serial_open(port_t *rp) {
 int serial_setup(port_t *rp)
 {
   int fd;
-#ifdef HAVE_TERMIOS_H
+  /* There's a lib replacement for termios under Mingw */
+#if defined(HAVE_TERMIOS_H) || defined(WIN32)
   speed_t speed;			/* serial comm speed */
   struct termios options;
 #elif defined(HAVE_TERMIO_H)
@@ -157,7 +174,7 @@ int serial_setup(port_t *rp)
    * Get the current options for the port...
    */
   
-#ifdef HAVE_TERMIOS_H
+#if defined(HAVE_TERMIOS_H) || defined(WIN32)
   tcgetattr(fd, &options);
 #elif defined(HAVE_TERMIO_H)
   ioctl (fd, TCGETA, &options);
@@ -204,7 +221,7 @@ int serial_setup(port_t *rp)
   default:
     rig_debug(RIG_DEBUG_ERR, "open_serial: unsupported rate specified: %d\n", 
 					rp->parm.serial.rate);
-	close(fd);
+	CLOSE(fd);
     return -RIG_ECONF;
   }
   /* TODO */
@@ -234,7 +251,7 @@ int serial_setup(port_t *rp)
   default:
     rig_debug(RIG_DEBUG_ERR,"open_serial: unsupported serial_data_bits "
 					"specified: %d\n", rp->parm.serial.data_bits);
-	close(fd);
+	CLOSE(fd);
     return -RIG_ECONF;
     break;
   }
@@ -256,7 +273,7 @@ int serial_setup(port_t *rp)
     rig_debug(RIG_DEBUG_ERR, "open_serial: unsupported serial_stop_bits "
 					"specified: %d\n",
 					rp->parm.serial.stop_bits);
-	close(fd);
+	CLOSE(fd);
     return -RIG_ECONF;
     break;
   }
@@ -282,7 +299,7 @@ int serial_setup(port_t *rp)
     rig_debug(RIG_DEBUG_ERR, "open_serial: unsupported serial_parity "
 					"specified: %d\n",
 					rp->parm.serial.parity);
-	close(fd);
+	CLOSE(fd);
     return -RIG_ECONF;
     break;
   }
@@ -310,7 +327,7 @@ int serial_setup(port_t *rp)
     rig_debug(RIG_DEBUG_ERR, "open_serial: unsupported flow_control "
 					"specified: %d\n",
 					rp->parm.serial.handshake);
-	close(fd);
+	CLOSE(fd);
     return -RIG_ECONF;
     break;
   }
@@ -319,7 +336,7 @@ int serial_setup(port_t *rp)
    * Choose raw input, no preprocessing please ..
    */
 
-#if defined(HAVE_TERMIOS_H) || defined(HAVE_TERMIO_H)
+#if defined(HAVE_TERMIOS_H) || defined(HAVE_TERMIO_H) || defined(WIN32)
   options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
 
   /*
@@ -343,25 +360,25 @@ int serial_setup(port_t *rp)
    * Finally, set the new options for the port...
    */
   
-#ifdef HAVE_TERMIOS_H
+#if defined(HAVE_TERMIOS_H) || defined(WIN32)
   if (tcsetattr(fd, TCSANOW, &options) == -1) {
 		rig_debug(RIG_DEBUG_ERR, "open_serial: tcsetattr failed: %s\n", 
 					strerror(errno));
-		close(fd);
+		CLOSE(fd);
 		return -RIG_ECONF;		/* arg, so close! */
   }
 #elif defined(HAVE_TERMIO_H)
   if (ioctl(fd, TCSETA, &options) == -1) {
 		rig_debug(RIG_DEBUG_ERR, "open_serial: ioctl(TCSETA) failed: %s\n", 
 					strerror(errno));
-		close(fd);
+		CLOSE(fd);
 		return -RIG_ECONF;		/* arg, so close! */
   }
 #else	/* sgtty */
   if (ioctl(fd, TIOCSETP, &sg) == -1) {
 		rig_debug(RIG_DEBUG_ERR, "open_serial: ioctl(TIOCSETP) failed: %s\n", 
 					strerror(errno));
-		close(fd);
+		CLOSE(fd);
 		return -RIG_ECONF;		/* arg, so close! */
   }
 #endif
@@ -390,7 +407,7 @@ int serial_flush( port_t *p )
  * assumes: p is not NULL
  */
 
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(WIN32)
 int ser_open(port_t *p)
 {
 	const char *path = p->pathname;
@@ -421,33 +438,105 @@ int ser_open(port_t *p)
 
 int ser_close(port_t *p)
 {
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(WIN32)
 		return CloseHandle(p->handle);
 #else
 		return close(p->fd);
 #endif
 }
 
-int ser_set_rts(const port_t *p, int state)
+int ser_set_rts(port_t *p, int state)
 {
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(WIN32)
 		/*
 		 * TODO: log error with 0x%lx GetLastError()
 		 */
 	return !EscapeCommFunction(p->handle, state ? SETRTS : CLRRTS);
 #else
-	unsigned char y = TIOCM_RTS;
+	unsigned int y = TIOCM_RTS;
+#if defined(TIOCMBIS) && defined(TIOCMBIC)
 	return ioctl(p->fd, state ? TIOCMBIS : TIOCMBIC, &y);
+#else
+	if (ioctl(p->fd, TIOCMGET, &y) < 0) {
+		return -RIG_EIO;
+	}
+	if (state)
+		y |= TIOCM_RTS;
+	else
+		y &= ~TIOCM_RTS;
+	return ioctl(p->fd, state ? TIOCMSET, &y);
+#endif
 #endif
 }
 
-int ser_set_dtr(const port_t *p, int state)
+/*
+ * assumes state not NULL
+ * p is supposed to be &rig->state.rigport
+ */
+int ser_get_rts(port_t *p, int *state)
 {
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(WIN32)
+	/* TODO... */
+  return -RIG_ENIMPL;
+#else
+  int status;
+  unsigned int y;
+  status = ioctl(p->fd, TIOCMGET, &y);
+  *state = (y & TIOCM_RTS) ? RIG_PTT_ON:RIG_PTT_OFF;
+  return RIG_OK;
+#endif
+}
+
+int ser_set_dtr(port_t *p, int state)
+{
+#if defined(WIN32)
 	return !EscapeCommFunction(p->handle, state ? SETDTR : CLRDTR);
 #else
-	unsigned char y = TIOCM_DTR;
+	unsigned int y = TIOCM_DTR;
+#if defined(TIOCMBIS) && defined(TIOCMBIC)
 	return ioctl(p->fd, state ? TIOCMBIS : TIOCMBIC, &y);
+#else
+	if (ioctl(p->fd, TIOCMGET, &y) < 0) {
+		return -RIG_EIO;
+	}
+	if (state)
+		y |= TIOCM_DTR;
+	else
+		y &= ~TIOCM_DTR;
+	return ioctl(p->fd, state ? TIOCMSET, &y);
+#endif
+#endif
+}
+
+int ser_get_dtr(port_t *p, int *state)
+{
+#if defined(WIN32)
+	/* TODO... */
+  return -RIG_ENIMPL;
+#else
+  int status;
+  unsigned int y;
+  status = ioctl(p->fd, TIOCMGET, &y);
+  *state = (y & TIOCM_DTR) ? RIG_PTT_ON:RIG_PTT_OFF;
+  return status;
+#endif
+}
+
+/*
+ * assumes state not NULL
+ * p is supposed to be &rig->state.rigport
+ */
+int ser_get_dcd(port_t *p, int *state)
+{
+#if defined(WIN32)
+	/* TODO... */
+  return -RIG_ENIMPL;
+#else
+  int status;
+  unsigned int y;
+  status = ioctl(p->fd, TIOCMGET, &y);
+  *state = (y & TIOCM_CAR) ? RIG_DCD_ON:RIG_DCD_OFF;
+  return RIG_OK;
 #endif
 }
 
@@ -476,29 +565,10 @@ int ser_ptt_get(port_t *p, ptt_t *pttx)
 {
 
 		switch(p->type.ptt) {
-#if defined(_WIN32) || defined(__CYGWIN__)
-				/* TODO... */
-#else
 		case RIG_PTT_SERIAL_RTS:
-			{
-				unsigned char y;
-				int status;
-
-				status = ioctl(p->fd, TIOCMGET, &y);
-				*pttx = y & TIOCM_RTS ? RIG_PTT_ON:RIG_PTT_OFF;
-				return status;
-			}
-
+			return ser_get_rts(p, &pttx);
 		case RIG_PTT_SERIAL_DTR:
-			{
-				unsigned char y;
-				int status;
-
-				status = ioctl(p->fd, TIOCMGET, &y);
-				*pttx = y & TIOCM_DTR ? RIG_PTT_ON:RIG_PTT_OFF;
-				return status;
-			}
-#endif
+			return ser_get_dtr(p, &pttx);
 		default:
 				rig_debug(RIG_DEBUG_ERR,"Unsupported PTT type %d\n",
 								p->type.ptt);
@@ -515,12 +585,12 @@ int ser_dcd_get(port_t *p, dcd_t *dcdx)
 {
 
 		switch(p->type.dcd) {
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(WIN32)
 				/* TODO... */
 #else
 		case RIG_DCD_SERIAL_CTS:
 			{
-				unsigned char y;
+				unsigned int y;
 				int status;
 
 				status = ioctl(p->fd, TIOCMGET, &y);
@@ -530,7 +600,7 @@ int ser_dcd_get(port_t *p, dcd_t *dcdx)
 
 		case RIG_DCD_SERIAL_DSR:
 			{
-				unsigned char y;
+				unsigned int y;
 				int status;
 
 				status = ioctl(p->fd, TIOCMGET, &y);
@@ -538,6 +608,8 @@ int ser_dcd_get(port_t *p, dcd_t *dcdx)
 				return status;
 			}
 #endif
+		case RIG_DCD_SERIAL_CAR:
+			return ser_get_dcd(p, &dcdx);
 		default:
 				rig_debug(RIG_DEBUG_ERR,"Unsupported DCD type %d\n",
 								p->type.dcd);
@@ -553,52 +625,216 @@ int ser_dcd_get(port_t *p, dcd_t *dcdx)
  * void parport_cleanup() { ioctl(fd, PPRELEASE); }
  */
 
-int par_open(port_t *p)
+int par_open(port_t *port)
 {
-		int fd;
+	int fd;
+	int mode;
+
+	if (!port->pathname)
+		return -RIG_EINVAL;
 
-		fd = open(p->pathname, O_RDWR);
-		if (fd < 0) {
-			rig_debug(RIG_DEBUG_VERBOSE, "Opening device \"%s\": %s\n", p->pathname, strerror(errno));
-			return -RIG_EIO;
-		}
 #ifdef HAVE_LINUX_PPDEV_H
-		if (ioctl(fd, PPCLAIM) < 0) {
-			rig_debug(RIG_DEBUG_VERBOSE, "Claiming device \"%s\": %s\n", p->pathname, strerror(errno));
-			close(fd);
-			return -RIG_EIO;
-		}
+	fd = open(port->pathname, O_RDWR);
+	if (fd < 0) {
+		rig_debug(RIG_DEBUG_ERR, "Opening device \"%s\": %s\n", port->pathname, strerror(errno));
+		return -RIG_EIO;
+	}
+	mode = IEEE1284_MODE_COMPAT;
+	if (ioctl (fd, PPSETMODE, &mode) != 0) {
+		rig_debug(RIG_DEBUG_ERR, "PPSETMODE \"%s\": %s\n", port->pathname, strerror(errno));
+		close(fd);
+		return -RIG_EIO;
+	}
+
+#elif defined(WIN32)
+	fd = (int)CreateFile(port->pathname, GENERIC_READ | GENERIC_WRITE,
+		0, NULL, OPEN_EXISTING, 0, NULL);
+	if (fd == (int)INVALID_HANDLE_VALUE) {
+		rig_debug(RIG_DEBUG_ERR, "Opening device \"%s\"\n", port->pathname);
+		CloseHandle((HANDLE)fd);
+		return -RIG_EIO;
+	}
+#else
+	return -RIG_ENIMPL;
 #endif
-		p->fd = fd;
-		return fd;
+	port->fd = fd;
+	return fd;
 }
 
-int par_close(port_t *p)
+int par_close(port_t *port)
 {
 #ifdef HAVE_LINUX_PPDEV_H
-		ioctl(p->fd, PPRELEASE);
+#elif defined(WIN32)
+	CloseHandle((HANDLE)(port->fd));
+	return RIG_OK;
 #endif
-		return close(p->fd);
+	return close(port->fd);
 }
 
+int par_write_data(port_t *port, unsigned char data)
+{
+#ifdef HAVE_LINUX_PPDEV_H
+	int status;
+	status = ioctl(port->fd, PPWDATA, &data);
+	return status == 0 ? RIG_OK : -RIG_EIO;
+#elif defined(WIN32)
+	unsigned int dummy;
+
+	if (!(DeviceIoControl((HANDLE)(port->fd), NT_IOCTL_DATA, &data, sizeof(data), 
+			NULL, 0, (LPDWORD)&dummy, NULL))) {
+		rig_debug(RIG_DEBUG_ERR, "%s: DeviceIoControl failed!\n", __FUNCTION__);
+	}
+#endif
+	return -RIG_ENIMPL;
+}
+
+int par_read_data(port_t *port, unsigned char *data)
+{
+#ifdef HAVE_LINUX_PPDEV_H
+	int status;
+	status = ioctl(port->fd, PPRDATA, data);
+	return status == 0 ? RIG_OK : -RIG_EIO;
+#elif defined(WIN32)
+	char ret;
+	unsigned int dummy;
+
+	if (!(DeviceIoControl((HANDLE)(port->fd), NT_IOCTL_STATUS, NULL, 0, &ret, 
+          		sizeof(ret), (LPDWORD)&dummy, NULL))) {
+		rig_debug(RIG_DEBUG_ERR, "%s: DeviceIoControl failed!\n", __FUNCTION__);
+	}
+
+  	return ret ^ S1284_INVERTED;
+#endif
+	return -RIG_ENIMPL;
+}
+
+
+int par_write_control(port_t *port, unsigned char control)
+{
+#ifdef HAVE_LINUX_PPDEV_H
+	int status;
+	unsigned char ctrl = control ^ CP_ACTIVE_LOW_BITS;
+	status = ioctl(port->fd, PPWCONTROL, &ctrl);
+	return status == 0 ? RIG_OK : -RIG_EIO;
+#elif defined(WIN32)
+  unsigned char ctr = control;
+  unsigned char dummyc;
+  unsigned int dummy;
+  const unsigned char wm = (C1284_NSTROBE |
+			    C1284_NAUTOFD |
+			    C1284_NINIT |
+			    C1284_NSELECTIN);
+
+  if (ctr & 0x20)
+    {
+      rig_debug (RIG_DEBUG_WARN, "use ieee1284_data_dir to change data line direction!\n");
+    }
+
+  /* Deal with inversion issues. */
+  ctr ^= wm & C1284_INVERTED;
+  ctr = (ctr & ~wm) ^ (ctr & wm);
+  if (!(DeviceIoControl((HANDLE)(port->fd), NT_IOCTL_CONTROL, &ctr, 
+          sizeof(ctr), &dummyc, sizeof(dummyc), (LPDWORD)&dummy, NULL))) {
+      rig_debug(RIG_DEBUG_ERR,"frob_control: DeviceIoControl failed!\n");
+  }
+  return RIG_OK;
+#endif
+	return -RIG_ENIMPL;
+}
+
+int par_read_control(port_t *port, unsigned char *control)
+{
+#ifdef HAVE_LINUX_PPDEV_H
+	int status;
+	unsigned char ctrl;
+	status = ioctl(port->fd, PPRCONTROL, &ctrl);
+	*control = ctrl ^ CP_ACTIVE_LOW_BITS;
+	return status == 0 ? RIG_OK : -RIG_EIO;
+#elif defined(WIN32)
+	char ret;
+	unsigned int dummy;
+
+	if (!(DeviceIoControl((HANDLE)(port->fd), NT_IOCTL_CONTROL, NULL, 0, &ret, 
+			sizeof(ret), (LPDWORD)&dummy, NULL))) {
+		rig_debug(RIG_DEBUG_ERR, "%s: DeviceIoControl failed!\n", __FUNCTION__);
+	}
+
+	*control = ret ^ S1284_INVERTED;
+#endif
+	return -RIG_ENIMPL;
+}
+
+
+int par_read_status(port_t *port, unsigned char *status)
+{
+#ifdef HAVE_LINUX_PPDEV_H
+	int ret;
+	unsigned char sta;
+	ret = ioctl(port->fd, PPRSTATUS, &sta);
+	*status = sta ^ SP_ACTIVE_LOW_BITS;
+	return ret == 0 ? RIG_OK : -RIG_EIO;
+#elif defined(WIN32)
+	unsigned char ret;
+	unsigned int dummy;
+
+	if (!(DeviceIoControl((HANDLE)(port->fd), NT_IOCTL_STATUS, NULL, 0, &ret, 
+			sizeof(ret), (LPDWORD)&dummy, NULL))) {
+		rig_debug(RIG_DEBUG_ERR, "%s: DeviceIoControl failed!\n", __FUNCTION__);
+	}
+
+	*status = ret ^ S1284_INVERTED;
+#endif
+	return -RIG_ENIMPL;
+}
+
+
+int par_lock(port_t *port)
+{
+#ifdef HAVE_LINUX_PPDEV_H
+	if (ioctl(port->fd, PPCLAIM) < 0) {
+		rig_debug(RIG_DEBUG_ERR, "Claiming device \"%s\": %s\n", port->pathname, strerror(errno));
+		return -RIG_EIO;
+	}
+	return RIG_OK;
+#elif defined(WIN32)
+	return RIG_OK;
+#endif
+	return -RIG_ENIMPL;
+}
+
+int par_unlock(port_t *port)
+{
+#ifdef HAVE_LINUX_PPDEV_H
+	if (ioctl(port->fd, PPRELEASE) < 0) {
+		rig_debug(RIG_DEBUG_ERR, "Releasing device \"%s\": %s\n", port->pathname, strerror(errno));
+		return -RIG_EIO;
+	}
+	return RIG_OK;
+#elif defined(WIN32)
+	return RIG_OK;
+#endif
+	return -RIG_ENIMPL;
+}
+
+
 int par_ptt_set(port_t *p, ptt_t pttx)
 {
 		switch(p->type.ptt) {
-#ifdef HAVE_LINUX_PPDEV_H
 		case RIG_PTT_PARALLEL:
 				{
 					unsigned char reg;
 					int status;
 
-					status = ioctl(p->fd, PPRDATA, &reg);
+					status = par_read_data(p, &reg);
+					if (status != RIG_OK)
+						return status;
 					if (pttx == RIG_PTT_ON)
 						reg |=   1 << p->parm.parallel.pin;
 					else
 						reg &= ~(1 << p->parm.parallel.pin);
 
-					return ioctl(p->fd, PPWDATA, &reg);
+					return par_write_data(p, reg);
 				}
-#endif
 		default:
 				rig_debug(RIG_DEBUG_ERR,"Unsupported PTT type %d\n", 
 								p->type.ptt);
@@ -613,18 +849,16 @@ int par_ptt_set(port_t *p, ptt_t pttx)
 int par_ptt_get(port_t *p, ptt_t *pttx)
 {
 		switch(p->type.ptt) {
-#ifdef HAVE_LINUX_PPDEV_H
 		case RIG_PTT_PARALLEL:
 				{
 					unsigned char reg;
 					int status;
 
-					status = ioctl(p->fd, PPRDATA, &reg);
+					status = par_read_data(p, &reg);
 					*pttx = reg & (1<<p->parm.parallel.pin) ? 
 							RIG_PTT_ON:RIG_PTT_OFF;
 					return status;
 				}
-#endif
 		default:
 				rig_debug(RIG_DEBUG_ERR,"Unsupported PTT type %d\n", 
 								p->type.ptt);
@@ -639,18 +873,16 @@ int par_ptt_get(port_t *p, ptt_t *pttx)
 int par_dcd_get(port_t *p, dcd_t *dcdx)
 {
 		switch(p->type.dcd) {
-#ifdef HAVE_LINUX_PPDEV_H
 		case RIG_DCD_PARALLEL:
 				{
 					unsigned char reg;
 					int status;
 
-					status = ioctl(p->fd, PPRDATA, &reg);
+					status = par_read_data(p, &reg);
 					*dcdx = reg & (1<<p->parm.parallel.pin) ? 
 							RIG_DCD_ON:RIG_DCD_OFF;
 					return status;
 				}
-#endif
 		default:
 				rig_debug(RIG_DEBUG_ERR,"Unsupported DCD type %d\n", 
 								p->type.dcd);
