@@ -2,7 +2,7 @@
  *  Hamlib Tentec backend - main file
  *  Copyright (c) 2001-2003 by Stephane Fillod
  *
- *	$Id: tentec.c,v 1.8 2003/04/16 22:30:43 fillods Exp $
+ *	$Id: tentec.c,v 1.12 2003/11/16 17:14:44 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
@@ -91,7 +92,6 @@ int tentec_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *
 int tentec_init(RIG *rig)
 {
 	struct tentec_priv_data *priv;
-	struct tentec_priv_caps *priv_caps;
 
 	priv = (struct tentec_priv_data*)malloc(sizeof(struct tentec_priv_data));
 
@@ -99,7 +99,6 @@ int tentec_init(RIG *rig)
 				/* whoops! memory shortage! */
 		return -RIG_ENOMEM;
 	}
-	priv_caps = (struct tentec_priv_caps *)rig->caps->priv;
 
 	memset(priv, 0, sizeof(struct tentec_priv_data));
 
@@ -112,7 +111,6 @@ int tentec_init(RIG *rig)
 	priv->cwbfo = 0;
 	priv->agc = 0.5;	/* medium */
 	priv->lnvol = priv->spkvol = 0.0;	/* mute */
-	memcpy(&priv->str_cal, &priv_caps->str_cal, sizeof(cal_table_t));
 
 	rig->state.priv = (rig_ptr_t)priv;
 
@@ -205,9 +203,12 @@ int tentec_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 	struct rig_state *rs = &rig->state;
 	int freq_len, retval;
 	char freqbuf[16];
+	freq_t old_freq;
 
 	priv = (struct tentec_priv_data *)rig->state.priv;
 
+	old_freq = priv->freq;
+	priv->freq = freq;
 	tentec_tuning_factor_calc(rig);
 
 	freq_len = sprintf(freqbuf, "N%c%c%c%c%c%c" EOM, 
@@ -216,10 +217,10 @@ int tentec_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 						priv->btf >> 8, priv->btf & 0xff);
 	
 	retval = write_block(&rs->rigport, freqbuf, freq_len);
-	if (retval != RIG_OK)
-			return retval;
-
-	priv->freq = freq;
+	if (retval != RIG_OK) {
+		priv->freq = old_freq;
+		return retval;
+	}
 
 	return RIG_OK;
 }
@@ -372,28 +373,30 @@ int tentec_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 {
 	struct tentec_priv_data *priv = (struct tentec_priv_data *)rig->state.priv;
 	int retval, lvl_len;
-	char lvlbuf[32];
+	unsigned char lvlbuf[32];
 
 
 	/* Optimize:
 	 *   sort the switch cases with the most frequent first
 	 */
 	switch (level) {
-	case RIG_LEVEL_STRENGTH:
+	case RIG_LEVEL_RAWSTR:
 			/* read A/D converted value */
 		lvl_len = 4;
 		retval = tentec_transaction (rig, "X" EOM, 2, lvlbuf, &lvl_len);
 		if (retval != RIG_OK)
 			return retval;
 
-		if (lvl_len != 4) {
+		if (lvl_len != 3) {
 			rig_debug(RIG_DEBUG_ERR,"tentec_get_level: wrong answer"
 							"len=%d\n", lvl_len);
 			return -RIG_ERJCTED;
 		}
 
-		lvlbuf[4] = '\0';
-		val->i = rig_raw2val((lvlbuf[1]<<8) + lvlbuf[1], &priv->str_cal);
+		lvlbuf[3] = '\0';
+		rig_debug(RIG_DEBUG_VERBOSE,"tentec_get_level: cmd=%c,hi=%d,lo=%d\n",
+				lvlbuf[0],lvlbuf[1],lvlbuf[2]);
+		val->i = (lvlbuf[1]<<8) + lvlbuf[2];
 		break;
 
 	case RIG_LEVEL_AGC:
@@ -442,11 +445,12 @@ const char *tentec_get_info(RIG *rig)
  */
 DECLARE_INITRIG_BACKEND(tentec)
 {
-		rig_debug(RIG_DEBUG_VERBOSE, "tentec: _init called\n");
+	rig_debug(RIG_DEBUG_VERBOSE, "tentec: _init called\n");
 
-		rig_register(&tt550_caps);
-		rig_register(&rx320_caps);
+	rig_register(&tt550_caps);
+	rig_register(&tt516_caps);
+	rig_register(&rx320_caps);
 
-		return RIG_OK;
+	return RIG_OK;
 }
 
