@@ -3,7 +3,7 @@
  * This programs dumps the capabilities of a backend rig.
  *
  *
- *    $Id: dumpcaps.c,v 1.37 2003/04/06 18:40:35 fillods Exp $  
+ *    $Id: dumpcaps.c,v 1.41 2003/12/04 23:18:49 fillods Exp $  
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -22,6 +22,9 @@
  * 
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -30,6 +33,8 @@
 #include <hamlib/rig.h>
 #include "misc.h"
 
+#include "sprintflst.h"
+
 
 static int print_ext(RIG *rig, const struct confparams *cfp, rig_ptr_t ptr);
 static const char *decode_mtype(chan_type_t type);
@@ -46,7 +51,7 @@ int dumpcaps (RIG* rig)
 	int status,i;
 	char freqbuf[20];
 	int backend_warnings=0;
-	char prntbuf[256];
+	static char prntbuf[1024];	/* a malloc would be better.. */
 
 	if (!rig || !rig->caps)
 		return -RIG_EINVAL;
@@ -132,6 +137,9 @@ int dumpcaps (RIG* rig)
 	case RIG_DCD_SERIAL_DSR:
 			printf("thru serial port (DTR/DSR)\n");
 			break;
+	case RIG_DCD_SERIAL_CAR:
+			printf("thru serial port (CD)\n");
+			break;
 	case RIG_DCD_NONE:
 			printf("None\n");
 			break;
@@ -145,6 +153,9 @@ int dumpcaps (RIG* rig)
 	case RIG_PORT_SERIAL:
 			printf("RS-232\n");
 			break;
+	case RIG_PORT_PARALLEL:
+			printf("Parallel\n");
+			break;
 	case RIG_PORT_DEVICE:
 			printf("device driver\n");
 			break;
@@ -211,10 +222,10 @@ int dumpcaps (RIG* rig)
 	sprintf_func(prntbuf, caps->has_set_func);
 	printf("Set functions: %s\n", prntbuf);
 
-	sprintf_level(prntbuf, caps->has_get_level);
+	sprintf_level_gran(prntbuf, caps->has_get_level, caps->level_gran);
 	printf("Get level: %s\n", prntbuf);
 
-	sprintf_level(prntbuf, caps->has_set_level);
+	sprintf_level_gran(prntbuf, caps->has_set_level, caps->level_gran);
 	printf("Set level: %s\n", prntbuf);
 	if (caps->has_set_level&RIG_LEVEL_READONLY_LIST) {
 			printf("Warning: backend can set readonly levels!\n");
@@ -224,10 +235,10 @@ int dumpcaps (RIG* rig)
 	rig_ext_level_foreach(rig, print_ext, NULL);
 	printf("\n");
 
-	sprintf_parm(prntbuf, caps->has_get_parm);
+	sprintf_parm_gran(prntbuf, caps->has_get_parm, caps->parm_gran);
 	printf("Get parameters: %s\n", prntbuf);
 
-	sprintf_parm(prntbuf, caps->has_set_parm);
+	sprintf_parm_gran(prntbuf, caps->has_set_parm, caps->parm_gran);
 	printf("Set parameters: %s\n", prntbuf);
 	if (caps->has_set_parm&RIG_PARM_READONLY_LIST) {
 			printf("Warning: backend can set readonly parms!\n");
@@ -237,20 +248,19 @@ int dumpcaps (RIG* rig)
 	rig_ext_parm_foreach(rig, print_ext, NULL);
 	printf("\n");
 
-#if 0
-	/* FIXME: use rig->state.vfo_list instead */
-	printf("VFO list: ");
-	if (caps->vfo_list!=0) {
-		if ((caps->vfo_list&RIG_VFO_A)==RIG_VFO_A) printf("VFOA ");
-		if ((caps->vfo_list&RIG_VFO_B)==RIG_VFO_A) printf("VFOB ");
-		if ((caps->vfo_list&RIG_VFO_C)==RIG_VFO_A) printf("VFOC ");
-		printf("\n");
-	} else {
-			printf(" none! This backend might be bogus!\n");
-	}
 
-	printf("Number of channels:\t%d\n", caps->chan_qty);
-#endif
+	if (rig->state.vfo_list!=0)
+		sprintf_vfo(prntbuf, rig->state.vfo_list);
+	else
+		strcpy(prntbuf," none! This backend might be bogus!\n");
+	printf("VFO list: %s\n", prntbuf);
+
+	sprintf_vfop(prntbuf, caps->vfo_ops);
+	printf("VFO Ops: %s\n", prntbuf);
+
+	sprintf_scan(prntbuf, caps->scan_ops);
+	printf("Scan Ops: %s\n", prntbuf);
+
 	printf("Number of banks:\t%d\n", caps->bank_qty);
 	printf("Memory name desc size:\t%d\n", caps->chan_desc_sz);
 
