@@ -5,7 +5,7 @@
  * It takes commands in interactive mode as well as 
  * from command line options.
  *
- * $Id: rigctl.c,v 1.42 2003/04/06 18:40:35 fillods Exp $  
+ * $Id: rigctl.c,v 1.49 2004/02/08 20:25:36 fillods Exp $  
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -33,13 +33,19 @@
 #include <string.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <errno.h>
 
 #include <getopt.h>
 
 #include <hamlib/rig.h>
 #include "misc.h"
+#include "sprintflst.h"
 
-
+#ifdef _WIN32
+#define LLFMT "L"
+#else
+#define LLFMT "ll"
+#endif
 
 #define MAXNAMSIZ 32
 #define MAXNBOPT 100	/* max number of different options */
@@ -140,6 +146,7 @@ declare_proto_rig(get_info);
 declare_proto_rig(dump_caps);
 declare_proto_rig(set_ant);
 declare_proto_rig(get_ant);
+declare_proto_rig(reset);
 
 
 
@@ -201,6 +208,7 @@ struct test_table test_list[] = {
 		{ 'z', "get_xit", get_xit, ARG_OUT, "XIT" },
 		{ 0x83, "set_ant", set_ant, ARG_IN, "Antenna" },
 		{ 0x84, "get_ant", get_ant, ARG_OUT, "Antenna" },
+		{ 0x85, "reset", reset, ARG_IN, "Reset" },
 		{ 0x00, "", NULL },
 
 };
@@ -257,6 +265,21 @@ char parse_arg(const char *arg)
 		return 0;
 }
 
+static int scanfc(const char *format, void *p)
+{
+	int ret;
+
+	do {
+		ret = scanf(format, p);
+		if (ret < 0) {
+			if (errno != EINTR)
+				rig_debug(RIG_DEBUG_ERR, "scanf: %s\n", strerror(errno));
+			continue;
+		}
+		return ret;
+	} while(1);
+}
+
 #define MAXCONFLEN 128
 
 int main (int argc, char *argv[])
@@ -353,6 +376,8 @@ int main (int argc, char *argv[])
 						dcd_type = RIG_DCD_SERIAL_DSR;
 					else if (!strcmp(optarg, "CTS"))
 						dcd_type = RIG_DCD_SERIAL_CTS;
+					else if (!strcmp(optarg, "CD"))
+						dcd_type = RIG_DCD_SERIAL_CAR;
 					else if (!strcmp(optarg, "PARALLEL"))
 						dcd_type = RIG_DCD_PARALLEL;
 					else if (!strcmp(optarg, "NONE"))
@@ -493,16 +518,17 @@ int main (int argc, char *argv[])
 				printf("\nRig command: ");
 
 				do {
-					scanf("%c", &cmd);
+					scanfc("%c", &cmd);
 
 					/* command by name */
 					if (cmd == '\\') {
 						unsigned char cmd_name[MAXNAMSIZ], *pcmd = cmd_name;
 						int c_len = MAXNAMSIZ;
 
-						scanf("%c", pcmd);
+						scanfc("%c", pcmd);
 						while(c_len-- && (isalnum(*pcmd) || *pcmd == '_' ))
-							scanf("%c", ++pcmd);
+							scanfc("%c", ++pcmd);
+
 						*pcmd = '\0';
 						cmd = parse_arg(cmd_name);
 						break;
@@ -523,7 +549,7 @@ int main (int argc, char *argv[])
 				/* comment line */
 				if (cmd == '#' || cmd == ';') {
 					while( cmd != '\n' && cmd != '\r')
-						scanf("%c", &cmd);
+						scanfc("%c", &cmd);
 					continue;
 				}
 				if (cmd == 'Q' || cmd == 'q')
@@ -553,7 +579,7 @@ int main (int argc, char *argv[])
 			if (!(cmd_entry->flags & ARG_NOVFO) && vfo_mode) {
 				if (interactive) {
 					printf("VFO: ");
-					scanf("%s", arg1);
+					scanfc("%s", arg1);
 					vfo = parse_vfo(arg1);
 				} else {
 					if (!argv[optind]) {
@@ -567,7 +593,7 @@ int main (int argc, char *argv[])
 			if ((cmd_entry->flags & ARG_IN1) && cmd_entry->arg1) {
 				if (interactive) {
 					printf("%s: ", cmd_entry->arg1);
-					scanf("%s", arg1);
+					scanfc("%s", arg1);
 					p1 = arg1;
 				} else {
 					if (!argv[optind]) {
@@ -581,7 +607,7 @@ int main (int argc, char *argv[])
 			if ((cmd_entry->flags & ARG_IN2) && cmd_entry->arg2) {
 				if (interactive) {
 					printf("%s: ", cmd_entry->arg2);
-					scanf("%s", arg2);
+					scanfc("%s", arg2);
 					p2 = arg2;
 				} else {
 					if (!argv[optind]) {
@@ -595,7 +621,7 @@ int main (int argc, char *argv[])
 			if ((cmd_entry->flags & ARG_IN3) && cmd_entry->arg3) {
 				if (interactive) {
 					printf("%s: ", cmd_entry->arg3);
-					scanf("%s", arg3);
+					scanfc("%s", arg3);
 					p3 = arg3;
 				} else {
 					if (!argv[optind]) {
@@ -704,6 +730,14 @@ static int print_conf_list(const struct confparams *cfp, rig_ptr_t data)
 				printf(", %s", cfp->u.c.combostr[i]);
 		printf("\n");
 		break;
+	case RIG_CONF_STRING:
+		printf("String.\n");
+		break;
+	case RIG_CONF_CHECKBUTTON:
+		printf("Check button.\n");
+		break;
+	default:
+		printf("Unkown conf\n");
 	}
 
 	return 1;  /* !=0, we want them all ! */
@@ -711,8 +745,8 @@ static int print_conf_list(const struct confparams *cfp, rig_ptr_t data)
 
 static int print_model_list(const struct rig_caps *caps, void *data)
 {
-	printf("%d\t%-14s%-16s%s\n", caps->rig_model, caps->mfg_name,
-										caps->model_name, caps->version);
+	printf("%d\t%-16s%-24s%-8s%s\n", caps->rig_model, caps->mfg_name,
+				caps->model_name, caps->version, strstatus(caps->status));
 	return 1;  /* !=0, we want them all ! */
 }
 
@@ -722,7 +756,7 @@ void list_models()
 
 	rig_load_all_backends();
 
-	printf("Rig#\tMfg           Model           Vers.\n");
+	printf("Rig#\tMfg             Model                   Vers.   Status\n");
 	status = rig_list_foreach(print_model_list, NULL);
 	if (status != RIG_OK ) {
 		printf("rig_list_foreach: error = %s \n", rigerror(status));
@@ -761,7 +795,7 @@ declare_proto_rig(set_freq)
 {
 		freq_t freq;
 
-		sscanf(arg1, "%lld", &freq);
+		sscanf(arg1, "%"FREQFMT, &freq);
 		return rig_set_freq(rig, vfo, freq);
 }
 
@@ -775,7 +809,7 @@ declare_proto_rig(get_freq)
 				return status;
 		if (interactive)
 			printf("%s: ", cmd->arg1); /* i.e. "Frequency" */
-		printf("%lld\n", freq);
+		printf("%lld\n", (long long)freq);
 		return status;
 }
 
@@ -830,7 +864,7 @@ declare_proto_rig(set_mode)
 		pbwidth_t width;
 
 		mode = parse_mode(arg1);
-		sscanf(arg2, "%d", (int*)&width);
+		sscanf(arg2, "%ld", &width);
 		return rig_set_mode(rig, vfo, mode, width);
 }
 
@@ -998,7 +1032,7 @@ declare_proto_rig(set_split_freq)
 {
 		freq_t txfreq;
 
-		sscanf(arg1, "%lld", &txfreq);
+		sscanf(arg1, "%"FREQFMT, &txfreq);
 		return rig_set_split_freq(rig, vfo, txfreq);
 }
 
@@ -1013,7 +1047,7 @@ declare_proto_rig(get_split_freq)
 				return status;
 		if (interactive)
 			printf("%s: ", cmd->arg1);
-		printf("%lld\n", txfreq);
+		printf("%lld\n", (long long)txfreq);
 		return status;
 }
 
@@ -1109,7 +1143,7 @@ declare_proto_rig(power2mW)
 		printf("Power [0.0 .. 1.0]: ");
 		scanf("%f", &power);
 		printf("Frequency: ");
-		scanf("%lld", &freq);
+		scanf("%"FREQFMT, &freq);
 		printf("Mode: ");
 		scanf("%d", &mode);
 		status = rig_power2mW(rig, &mwp, power, freq, mode);
@@ -1413,7 +1447,7 @@ declare_proto_rig(get_channel)
 
 static int myfreq_event(RIG *rig, vfo_t vfo, freq_t freq, rig_ptr_t arg)
 {
-	printf("Event: freq changed to %lliHz on %s\n", freq, strvfo(vfo));
+	printf("Event: freq changed to %lliHz on %s\n", (long long)freq, strvfo(vfo));
 	return 0;
 }
 
@@ -1446,7 +1480,14 @@ declare_proto_rig(set_trn)
 {
 	int trn;
 
-	sscanf(arg1, "%d", &trn);
+	if (!strcmp(arg1, "OFF"))
+		trn = RIG_TRN_OFF;
+	else if (!strcmp(arg1, "RIG") || !strcmp(arg1, "ON"))
+		trn = RIG_TRN_RIG;
+	else if (!strcmp(arg1, "POLL"))
+		trn = RIG_TRN_POLL;
+	else
+		return -RIG_EINVAL;
 
 	if (trn != RIG_TRN_OFF) {
 		rig_set_freq_callback(rig, myfreq_event, NULL);
@@ -1462,16 +1503,16 @@ declare_proto_rig(set_trn)
 
 declare_proto_rig(get_trn)
 {
-		int status;
-		int trn;
+	int status;
+	int trn;
 
-		status = rig_get_trn(rig, &trn);
-		if (status != RIG_OK)
-				return status;
-		if (interactive)
-			printf("%s: ", cmd->arg1);
-		printf("%d\n", trn);
+	status = rig_get_trn(rig, &trn);
+	if (status != RIG_OK)
 		return status;
+	if (interactive)
+		printf("%s: ", cmd->arg1);
+	printf("%d\n", trn);
+	return status;
 }
 
 declare_proto_rig(get_info)
@@ -1496,7 +1537,7 @@ void dump_chan(RIG *rig, channel_t *chan)
 	printf("Channel: %d, Name: '%s'\n", chan->channel_num, 
 					chan->channel_desc);
 
-	printf("VFO: %s, Antenna: %d, Split %s\n", strvfo(chan->vfo),
+	printf("VFO: %s, Antenna: %d, Split: %s\n", strvfo(chan->vfo),
 					chan->ant, chan->split==RIG_SPLIT_ON?"ON":"OFF");
 
 	sprintf_freq(freqbuf, chan->freq);
@@ -1579,4 +1620,12 @@ declare_proto_rig(get_ant)
 	return status;
 }
 
+declare_proto_rig(reset)
+{
+	reset_t reset;
+
+	sscanf(arg1, "%d", &reset);
+	return rig_reset(rig, reset);
+}
+
 
