@@ -6,7 +6,7 @@
  * via serial interface to an FT-847 using the "CAT" interface.
  *
  *
- * $Id: ft847.c,v 1.22 2003/04/07 22:42:05 fillods Exp $  
+ * $Id: ft847.c,v 1.27 2004/02/16 21:38:59 fillods Exp $  
  *
  *
  *
@@ -286,6 +286,7 @@ const struct rig_caps ft847_caps = {
       /* mode/filter list, .remember =  order matters! */
 .filters =  {
 		{RIG_MODE_SSB|RIG_MODE_CW, kHz(2.2)},
+		{RIG_MODE_CW, Hz(500)},
 		{RIG_MODE_AM, kHz(9)},
 		{RIG_MODE_AM, kHz(2.2)},
 		{RIG_MODE_FM, kHz(15)},
@@ -460,23 +461,15 @@ int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq) {
    */
 
 
-  rig_debug(RIG_DEBUG_VERBOSE,"ft847: vfo =%i \n", vfo);
+  rig_debug(RIG_DEBUG_VERBOSE,"ft847: vfo =%s \n", strvfo(vfo));
   
+  if (vfo == RIG_VFO_CURR)
+	vfo = p->current_vfo;
+
   switch(vfo) {
   case RIG_VFO_MAIN:
     cmd_index = FT_847_NATIVE_CAT_SET_FREQ_MAIN;
     break;
-  case RIG_VFO_CURR:
-    switch(p->current_vfo) {	/* what is my active VFO ? */
-    case RIG_VFO_MAIN:
-      cmd_index = FT_847_NATIVE_CAT_SET_FREQ_MAIN;
-      break;
-    default:
-      rig_debug(RIG_DEBUG_VERBOSE,"ft847: Unknown default VFO \n");
-      return -RIG_EINVAL;		/* sorry, wrong current VFO */
-    }
-    break;
-
   default:
     rig_debug(RIG_DEBUG_VERBOSE,"ft847: Unknown  VFO \n");
     return -RIG_EINVAL;		/* sorry, wrong VFO */
@@ -517,10 +510,10 @@ static int get_freq_and_mode(RIG *rig, vfo_t vfo, freq_t *freq, rmode_t *mode,
 
   p = (struct ft847_priv_data*)rs->priv;
 
-  rig_debug(RIG_DEBUG_VERBOSE,"ft847: vfo =%i \n", vfo);
+  rig_debug(RIG_DEBUG_VERBOSE,"ft847: vfo =%s \n", strvfo(vfo));
   
-  if (vfo == RIG_VFO_MAIN)
-		  vfo = p->current_vfo;
+  if (vfo == RIG_VFO_CURR)
+	vfo = p->current_vfo;
 
   /*
    * TODO:
@@ -562,13 +555,11 @@ static int get_freq_and_mode(RIG *rig, vfo_t vfo, freq_t *freq, rmode_t *mode,
 			   *mode = RIG_MODE_CW;
 			   break;
 
-#ifdef RIG_MODE_CWR
   case MD_CWNR:
 			   *width = rig_passband_narrow(rig, RIG_MODE_CW);
   case MD_CWR: 
 				*mode = RIG_MODE_CWR;
 				break;
-#endif
 
   case MD_AMN:
 			   *width = rig_passband_narrow(rig, RIG_MODE_AM);
@@ -592,7 +583,7 @@ static int get_freq_and_mode(RIG *rig, vfo_t vfo, freq_t *freq, rmode_t *mode,
 }
 
 /*
- * Note taken from http://my.en.com/~rayd/ft-847/FAQ/page4.htm#pollingcodes
+ * Note taken from http://www.supercontrol.de/cat/ft847faq/page4.htm#pollingcodes
  * The FT-847, as originally delivered, could not poll the radio for frequency
  * and mode information. This was added beginning with the 8G05 production
  * runs. The Operating Manual does not show the codes for polling the radio.
@@ -616,6 +607,11 @@ int ft847_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 
 int ft847_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width) {
   unsigned char cmd_index;	/* index of sequence to send */
+  struct rig_state *rs = &rig->state;
+  struct ft847_priv_data *p;
+
+  p = (struct ft847_priv_data*)rs->priv;
+
   
   /* 
    * translate mode from generic to ft847 specific
@@ -623,6 +619,15 @@ int ft847_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width) {
 
   rig_debug(RIG_DEBUG_VERBOSE,"ft847: generic mode = %x \n", mode);
 
+  if (vfo == RIG_VFO_CURR)
+	vfo = p->current_vfo;
+
+  if (vfo != RIG_VFO_MAIN) {
+    rig_debug(RIG_DEBUG_VERBOSE,"ft847: unsupported VFO '%s'\n", strvfo(vfo));
+    return -RIG_ENIMPL;		/* sorry, it's in TODO list */
+  }
+
+
   switch(mode) {
   case RIG_MODE_AM:
     cmd_index = FT_847_NATIVE_CAT_SET_MODE_MAIN_AM;
@@ -718,6 +723,9 @@ int ft847_set_vfo(RIG *rig, vfo_t vfo) {
    */
 
   switch(vfo) {
+  case RIG_VFO_CURR:
+	  return 0;		/* easy! */
+
   case RIG_VFO_MAIN:
     p->current_vfo = vfo;		/* update active VFO */
     break;
@@ -775,19 +783,3 @@ int ft847_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt) {
 
 }
 
-
-
-/*
- * init_ft847 is called by rig_backend_load
- */
-
-int init_ft847(void *be_handle) {
-  rig_debug(RIG_DEBUG_VERBOSE, "ft847: _init called\n");
-
-  rig_register(&ft847_caps);
-  
-  return RIG_OK;
-}
-
-
-
